﻿if not myBigIgnite then
	return
end


local MBI = myBigIgnite
local L = LibStub("AceLocale-3.0"):GetLocale("myBigIgnite", true)
local LSM = LibStub("LibSharedMedia-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")
MBI.cfMode = false

-- cache
local pairs, ipairs, type, tonumber, tostring, print, tinsert, tremove, wipe = pairs, ipairs, type, tonumber, tostring, print, tinsert, tremove, wipe
local strfind, strsub, floor, ceil, format = strfind, strsub, floor, ceil, format
local _



--##############################################################################################################################################
-- house keeping functions ---------------------------------------------------------------------------------------------------------------------

local function DeepCopy(src)
	local res = {}
    for k, v in pairs(src) do
        if type(v) == "table" then
            v = DeepCopy(v)
        end
        res[k] = v
    end
    return res
end

local function IsInCombat()
	return MBI.inCombat
end

local function Round(number, dec)
	return math.floor(number * 10^dec + 0.5) / 10^dec
end

local function FormatText(txt)
	if type(txt) == "number" then
		if MBI.db.profile.ignite.layout.text.format == "condensed" then
			txt = Round(txt / 1000, 1) .. "k"
		end
	end
	return txt
end

local function GetColor(t)
	return t.r, t.g, t.b, t.a
end

local function SetColor(t, r, g, b, a)
	t.r, t.g, t.b, t.a = r, g, b, a or t.a
end

local function List(mediatype)
	local t = {}
	for k, v in pairs(LSM:List(mediatype)) do
		t[v] = v
	end
	return t
end

--[[
This function walks trough a "maze" table following the path provided by the map table.
Read: the value at the end of the path is returned by the function.
Write: the new value must be passed to the newValue parameter. The value and the new value must be of the same type otherwise the newValue is discarded.
i is used as a recursive increment to walk through the table and shouldn't be explicitly passed to the function unless for specific purposes.
]]
local function TableWalk(maze, map, newValue, i)
	i = i or 1
	local value

	-- debug
	if type(maze[map[i]]) ~= "boolean" then
		assert(maze[map[i]], "There is no entry in the maze table for the step number " .. i .. " with the name \"" .. map[i] .. "\".\n" )
	end

	-- recursive calls are stoped if either the end of the maze table or the end of the map table is reached
	if type(maze[map[i]]) ~= "table" or i >= #map then
		if newValue ~= nil then
			maze[map[i]] = newValue
		end
		value = maze[map[i]]
	else
		value = TableWalk(maze[map[i]], map, newValue, i + 1)
	end
	return value
end

local function HlString(str, colorL, colorR)
	local colorL = colorL or MBI.ORANGE
	local colorR = colorR or MBI.NORMAL
	return colorL .. str .. colorR
end

local function IsPositiveNum(info, value)
	if tonumber(value) ~= nil and tonumber(value)> 0 then
		return true
	else
		print(format(L["%senter a positive number"], HlString(L["myBigIgnite: "])))
	end
end



--##############################################################################################################################################
-- option tables -------------------------------------------------------------------------------------------------------------------------------

-- default option table
function MBI:LoadFakeOptions()
	MBI.fakeOptions = {
		type = "group",
		name = "myBigIgnite",
		childGroups = "tab",
		disabled = IsInCombat,
		args = {
			toggleOptions = {
				type = "execute",
				name = L["myBigIgnite config"],
				desc = L["Toggles myBigIgnite configuration panel."],
				order = 1,
				func = function()
					MBI:ChangeConfigDisplay("fireSpec")
					MBI.configLoaded = true
				end,
			},
		},
	}

	-- ACE registry
	MBI.options = DeepCopy(MBI.fakeOptions)
	ACR:RegisterOptionsTable("myBigIgnite", MBI.options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("myBigIgnite", "myBigIgnite")
end

-- config panel, switching out of fire spec
function MBI:LoadooFOptions()
	MBI.ooFOptions = {
		type = "group",
		name = "myBigIgnite",
		childGroups = "tab",
		args = {
			message = {
				type = "description",
				name = L["Configuration available only when myBigIgnite is enabled."],
				fontSize = "medium",
			},
		},
	}
end

-- real config panel
function MBI:LoadFireOptions()
	MBI.fireOptions = {
		type = "group",
		name = "myBigIgnite",
		childGroups = "tab",
		disabled = IsInCombat,
		args = {
			configMode = {
				type = "toggle",
				name = L["Configuration mode"],
				desc = L["Enable myBigIgnite configuration mode."],
				order = 0,
				get = function()
					return MBI.cfMode
				end,
				set = function(info, value)
					MBI.cfMode = value
					MBI:ConfigMode(value)
				end,
			},
			stats = {
				type = "execute",
				name = L["Stats"],
				desc = L["Shows damages statistics."],
				order = 3,
				func = function()
					MBI:OpenInfo("stats")
				end,
			},
			help = {
				type = "execute",
				name = L["Help"],
				desc = L["Shows help."],
				order = 7,
				func = function()
					MBI:OpenInfo("help")
				end,
			},
			reset = {
				type = "execute",
				name = L["Reset"],
				desc = L["Reset all settings."],
				order = 10,
				confirm = true,
				width = "half",
				confirmText = L["This will reset all myBigIgnite settings. Are you sure?"],
				func = function()
					local temp = DeepCopy(MBI.defaults.profile)
					for k, v in pairs(MBI.db.profile) do
						if MBI.defaults.profile[k] then
							if type(MBI.db.profile[k]) == "table" then
								wipe(MBI.db.profile[k])
							end
							MBI.db.profile[k] = temp[k]
						end
					end
					MBI:RefreshConfig()
					print(MBI.YELLOW_LIGHT .. L["myBigIgnite has been reset."])
				end,
			},
			threshold = {
				type = "group",
				name = L["Threshold"],
				desc = L["Ignite threshold options."],
				order = 20,
				get = function(info) return TableWalk(MBI.db.profile, info)	end,
				set = function(info, value)
					TableWalk(MBI.db.profile, info, value)
				end,
				args = MBI:ThresholdOptions(),
			},
			general = {
				type = "group",
				name = L["General"],
				desc = L["General options."],
				order = 30,
				get = function(info) return TableWalk(MBI.db.profile, info)	end,
				set = function(info, value)
					TableWalk(MBI.db.profile, info, value)
					MBI.font = LSM:Fetch("font", MBI.db.profile.general.layout.font)
					MBI:SetIgniteFrame()
				end,
				args = MBI:GeneralOptions(),
			},
			ignite = {
				type = "group",
				name = L["Ignite frame"],
				desc = L["Ignite frame options."],
				order = 40,
				get = function(info) return TableWalk(MBI.db.profile, info)	end,
				set = function(info, value)
					TableWalk(MBI.db.profile, info, value)
					MBI:SetIgniteFrame()
				end,
				args = MBI:IgniteOptions(),
			},




		},
	}
end

-- general tab
function MBI:GeneralOptions()
	local UIScale = UIParent:GetScale()
	local UIWidth = UIParent:GetWidth()
	local UIHeight = UIParent:GetHeight()

	local options = {
		autohide = {
			type = "group",
			inline = true,
			order = 0,
			args = {
				header = {
					type = "header",
					name = L["Autohiding"],
					order = 0,
				},
				desc = {
					type = "description",
					name = MBI.YELLOW .. L["Hide automatically myBigIgnite if:"],
					fontSize = "medium",
					width = "full",
					order = 10,
				},
				noCombat = {
					type = "toggle",
					name = L["Out of combat"],
					order = 20,
					set = function(info, value)
						MBI.db.profile.general.autohide.noCombat = value
						if not MBI.cfMode then
							if value then
								MBI.F.main:Hide()
							else
								MBI.F.main:Show()
							end
						end
						if not value then
							MBI.db.profile.general.autohide.noIgnite = false
							MBI.db.profile.general.autohide.combuCD = false
						end
					end,
				},
				noIgnite = {
					type = "toggle",
					name = L["No Ignite"],
					order = 30,
					set = function(info, value)
						MBI.db.profile.general.autohide.noIgnite = value
						if value then
							MBI.db.profile.general.autohide.noCombat = true
							if not MBI.cfMode then
								if value then
									MBI.F.main:Hide()
								else
									MBI.F.main:Show()
								end
							end
						end
					end,
				},
				space1 = {
					type = "description",
					name = " ",
					width = "full",
					order = 40,
				},
				combuCD = {
					type = "toggle",
					name = L["Combustion in CD"],
					order = 50,
					set = function(info, value)
						MBI.db.profile.general.autohide.combuCD = value
						if value then
							MBI.db.profile.general.autohide.noCombat = true
							if not MBI.cfMode then
								if value then
									MBI.F.main:Hide()
								else
									MBI.F.main:Show()
								end
							end
						end
					end,
				},
				combuCDTime = {
					type = "range",
					name = L["Time before CD end"],
					desc = L["Shows the frame when Combustion CD is less than this value."],
					order = 60,
					disabled = function()
						return not MBI.db.profile.general.autohide.combuCD
					end,
					min = 0, max = 45, softMax = 30, step = 1,
				},
				space = {
					type = "description",
					name = " ",
					width = "full",
					order = 500,
				},
			},
		},
		anchor = {
			type = "group",
			inline = true,
			order = 10,
			args = {
				header = {
					type = "header",
					name = L["Position"],
					order = 0,
				},
				coord = {
					type = "group",
					inline = true,
					order = 10,
					args = {
						xcd = {
							type = "range",
							name = L["Horizontal position"],
							order = 0,
							min = 0,
							max = Round(UIWidth, 0),
							softMin = 0,
							softMax = Round(UIWidth, 0),
							bigStep = 0.5,
						},
						ycd = {
							type = "range",
							name = L["Vertical position"],
							order = 10,
							min = 0,
							max = Round(UIHeight, 0),
							softMin = 0,
							softMax = Round(UIHeight, 0),
							bigStep = 0.5,
						},
						resetPosition = {
							type = "execute",
							name = L["Reset position"],
							desc = L["Reset the main frame to its default position."],
							order = 20,
							confirm = true,
							confirmText = L["Reset the main frame position?"],
							func = function()
								MBI.db.profile.general.anchor.coord = DeepCopy(MBI.defaults.profile.general.anchor.coord)
								MBI:SetAnchorFramePosition()
							end,
						},
					},
				},
				space1 = {
					type = "description",
					name = " ",
					width = "full",
					order = 20,
				},
				align = {
					type = "select",
					name = L["Main Window alignment"],
					order = 30,
					values = {
						RIGHT = L["Right"],
						LEFT = L["Left"],
						CENTER = L["Center"],
						TOP = L["Top"],
						BOTTOM = L["Bottom"],
						TOPLEFT = L["TopLeft"],
						BOTTOMLEFT = L["BottomLeft"],
						TOPRIGHT = L["TopRight"],
						BOTTOMRIGHT = L["BottomRight"],
					},
				},
				strata = {
					type = "select",
					name = L["Window strata"],
					order = 40,
					values = {
							[1] = L["Low"],
							[2] = L["Medium"],
							[3] = L["High"],
					},
				},
				space = {
					type = "description",
					name = " ",
					width = "full",
					order = 500,
				},
			},
		},
		layout = {
			type = "group",
			inline = true,
			order = 20,
			args = {
				header = {
					type = "header",
					name = L["Layout"],
					order = 0,
				},
				textSize = {
					type = "range",
					name = L["Text and window size"],
					order = 10,
					min = 8, max = 20, step = 1,
				},
				alpha = {
					type = "range",
					order = 20,
					name = L["Main window alpha"],
					min = 0.4, max = 1, bigStep = 0.05,
				},
				space1 = {
					type = "description",
					name = " ",
					width = "full",
					order = 30,
				},
				font = {
					type = "select",
					name = L["Font"],
					order = 40,
					values = function()
						return List("font")
					end,
				},
				bar = {
					type = "select",
					order = 50,
					disabled = function()
						return not MBI.db.profile.ignite.timer.enabled and not MBI.db.profile.ignite.power.enabled
					end,
					name = L["Bar"],
					values = List("statusbar"),
				},
				space2 = {
					type = "description",
					name = " ",
					width = "full",
					order = 60,
				},
				background = {
					type = "select",
					name = L["Background"],
					order = 70,
					values = function()
						return List("background")
					end,
				},
				border = {
					type = "select",
					name = L["Border"],
					order = 80,
					values = function()
						return List("background")
					end,
				},
				space = {
					type = "description",
					name = " ",
					width = "full",
					order = 500,
				},
			},
		},
		header1 = {
			type = "header",
			name = L["Miscellaneous"],
			order = 30,
		},
		splashMsg = {
			type = "toggle",
			name = L["Splash message"],
			desc = L["Enables the welcome message on myBigIgnite loading."],
			order = 40,
		},
		resetStats = {
			type = "range",
			name = L["Auto reset of stats"],
			desc = L["(every x million of ticks). Simply to avoid storing super huge numbers."],
			order = 50,
			min = 0.1, max = 1, step = 0.1,
		},

		-----------------------TEMP--------------------
		auraHeader = {
			type = "header",
			name = L["DoT tracking (temporary)"],
			order = 100,
		},
		pyroblast = {
			type = "toggle",
			name = L["pyroblast"],
			desc = L["Toggle to activate Pyroblast dot tracking."],
			order = 200,
			width = "full",
			get = function ()
				return MBI.db.profile.trackpyro
			end,
			set = function (info, value)
				MBI.db.profile.trackpyro = value
				MBI:CreateSpellTables()
			end,
		},
		-----------------------TEMP--------------------
	}

	return options
end

-- threshold tab
function MBI:ThresholdOptions()
	local options = {
		descHigh = {
			type = "description",
			name = MBI.ORANGE .. L["HIGH IGNITE TICK: minimal value above which myBigIgnite will inform you that Ignite tick is optimal."],
			order = 10,
			width = "full",
			fontSize = "medium",
		},
		value = {
			type = "input",
			name = L["Ignite threshold value"],
			order = 20,
			validate = IsPositiveNum,
			get = function()
				return tostring(MBI.db.profile.threshold.value)
			end,
			set = function(info, value)
				MBI.db.profile.threshold.value = tonumber(value)
				MBI.db.profile.stats.info.thresholdCount = 0
				MBI.db.profile.stats.info.supOptT = 0
				MBI.db.profile.stats.info.supIntermT = 0
				if MBI.F.info and MBI.F.info:IsShown() and MBI.infoTable.type == "stats" then
					MBI:PopulateInfo("stats")
					MBI:InfoScrollUpdate()
				end
				if MBI.warning then
					MBI.warning = false
				end
			end,
		},
		high = {
			type = "group",
			inline = true,
			order = 25,
			args = {
				color = {
					type = "color",
					order = 0,
					name = L["Colour scheme"],
					get = function()
						return GetColor(MBI.db.profile.threshold.high.color)
					end,
					set = function(info, r, g, b)
						SetColor(MBI.db.profile.threshold.high.color, r, g, b)
					end,
				},
				mult = {
					type = "range",
					name = L["Scale multiplier"],
					desc = L["Main window is enlarged by this setting when the corresponding threshold is reached."],
					order = 10,
					min = 1, max = 3, softMin = 1, softMax = 2, step = 0.1,
				},
			},
		},
		space1 = {
			type = "description",
			name = " ",
			width = "full",
			order = 30,
		},
		space2 = {
			type = "description",
			name = " ",
			widh = "full",
			order = 40,
		},
		interm = {
			type = "group",
			inline = true,
			order = 50,
			args = {
				descMid = {
					type = "description",
					name = MBI.ORANGE .. L["MID IGNITE TICK: intermediate value, under optimal value. Useful in situations where you cannot expect to get a big Ignite."],
					order = 0,
					width = "full",
					fontSize = "medium",
				},
				enabled = {
					type = "toggle",
					name = L["Intermediate threshold"],
					desc = L["Enable intermediate threshold"],
					order = 10,
				},
				igPerctAbs = {
					type = "input",
					name = " ",
					order = 20,
					width = "half",
					disabled = true,
					get = function()
						if MBI.db.profile.threshold.interm.enabled then
							return "|cffFFFFFF" .. tostring(MBI.db.profile.threshold.interm.perct * MBI.db.profile.threshold.value)
						end
					end,
				},
				perct = {
					type = "range",
					name = L["% of optimal Ignite"],
					order = 30,
					disabled = function()
						return not MBI.db.profile.threshold.interm.enabled
					end,
					min = 0.05,	max = 0.99,	softMin = 0.5, softMax = 0.95, step = 0.05,
					isPercent = true,
				},
			},
		},
		mid = {
			type = "group",
			inline = true,
			order = 60,
			args = {
				color = {
					type = "color",
					order = 0,
					disabled = function()
						return not MBI.db.profile.threshold.interm.enabled
					end,
					name = L["Colour scheme"],
					get = function()
						return GetColor(MBI.db.profile.threshold.mid.color)
					end,
					set = function(info, r, g, b)
						SetColor(MBI.db.profile.threshold.mid.color, r, g, b)
					end,
				},
				mult = {
					type = "range",
					disabled = function()
						return not MBI.db.profile.threshold.interm.enabled
					end,
					name = L["Scale multiplier"],
					desc = L["Main window is enlarged by this setting when the corresponding threshold is reached."],
					order = 10,
					min = 1, max = 3, softMin = 1, softMax = 2, step = 0.1,
				},
			},
		},
		space4 = {
			type = "description",
			name = " ",
			widh = "full",
			order = 70,
		},
		space5 = {
			type = "description",
			name = " ",
			widh = "full",
			order = 80,
		},
		descLow = {
			type = "description",
			name = MBI.ORANGE .. L["LOW IGNITE TICK: default display when Ignite tick doesn't rich a sufficient level or Combution is not available."],
			order = 85,
			width = "full",
			fontSize = "medium",
		},
		low = {
			type = "group",
			inline = true,
			order = 90,
			args = {
				color = {
					type = "color",
					order = 0,
					name = L["Colour scheme"],
					get = function()
						return GetColor(MBI.db.profile.threshold.low.color)
					end,
					set = function(info, r, g, b)
						SetColor(MBI.db.profile.threshold.low.color, r, g, b)
					end,
				},
			},
		},
	}

	return options
end

-- ignite tab
function MBI:IgniteOptions()
	local options = {
		header = {
			type = "header",
			name = L["Ignite damage display"],
			order = 0,
		},
		useCombatLog = {
			type = "toggle",
			name = L["Use combat log"],
			desc = L["If this option is enabled, myBigIgnite will fetch Ignite damages from combat log instead of using the new method based on buffs. Consequently, damages will be displayed after each tick of Ignite as previously. (Not recommended)"],
			order = 10,
			width = "full",
		},
		smoothDmgReport = {
			type = "toggle",
			name = L["Smoothing damage"],
			desc = L["Smoothes damage display: this option introduces a lag (in seconds) before damage display is refreshed. This allows to even the display without compromising its accuracy."],
			order = 20,
			disabled = function()
				return MBI.db.profile.ignite.useCombatLog
			end,
		},
		smoothDmgValue = {
			type = "range",
			name = L["Lag time"],
			desc = L["In seconds."],
			order = 30,
			disabled = function()
				return not MBI.db.profile.ignite.smoothDmgReport or MBI.db.profile.ignite.useCombatLog
			end,
			min = 0.2, max = 1, softMin = 0.2, softMax = 1, step = 0.1,
		},
		dmgAsDps = {
			type = "toggle",
			name = L["Show DPS"],
			desc = L["Shows Ignite damage as DPS instead of raw tick damage."],
			order = 40,
			width = "full",
		},
		space4 = {
			type = "description",
			name = " ",
			widh = "full",
			order = 50,
		},
		timer = {
			type = "group",
			inline = true,
			order = 70,
			args = {
				header = {
					type = "header",
					name =  L["Ignite timer"],
					order = 0,
				},
				enabled = {
					type = "toggle",
					order = 10,
					name = L["Enable"],
					desc = L["Activates a timer bar showing visually the remaining time of an Ignite dot."],
				},
				space1 = {
					type = "description",
					name = " ",
					width = "full",
					order = 20,
				},
				reverse = {
					type = "toggle",
					order = 30,
					disabled = function()
						return not MBI.db.profile.ignite.timer.enabled
					end,
					name = L["Reversed"],
				},
				tick = {
					type = "toggle",
					order = 35,
					disabled = function()
						return not MBI.db.profile.ignite.timer.enabled
					end,
					name = L["Enable tick marks"],
				},
				alpha = {
					type = "range",
					order = 50,
					disabled = function()
						return not MBI.db.profile.ignite.timer.enabled
					end,
					name = L["Alpha"],
					min = 0.2, max = 1, bigStep = 0.05,
				},
				space4 = {
					type = "description",
					name = " ",
					width = "full",
					order = 70,
				},
				redZone = {
					type = "range",
					order = 80,
					disabled = function()
						return not MBI.db.profile.ignite.timer.enabled
					end,
					name = L["Ignite red zone"],
					desc = L["The timer bar will turn of the selected colour when the remaining time of Ignite is under this value."],
					min = 0, max = 2, step = 0.05, bigStep = 0.1,
				},
				redZoneColor = {
					type = "color",
					order = 90,
					disabled = function()
						return not MBI.db.profile.ignite.timer.enabled
					end,
					name = L["Red zone colour"],
					get = function()
						return GetColor(MBI.db.profile.ignite.timer.redZoneColor)
					end,
					set = function(info, r, g, b)
						SetColor(MBI.db.profile.ignite.timer.redZoneColor, r, g, b)
					end,
				},
				space = {
					type = "description",
					name = " ",
					width = "full",
					order = 500,
				},
			},
		},
		power = {
			type = "group",
			inline = true,
			order = 80,
			args = {
				header = {
					type = "header",
					name =  L["Ignite power bar"],
					order = 0,
				},
				enabled = {
					type = "toggle",
					order = 10,
					name = L["Enable"],
					desc = L["Activates a power bar showing visually how high is your current ignite tick."],
				},
				space1 = {
					type = "description",
					name = " ",
					width = "full",
					order = 15,
				},
				align = {
					type = "select",
					disabled = function()
						return not MBI.db.profile.ignite.power.enabled
					end,
					name = L["Bar alignement"],
					order = 20,
					values = {
						RIGHT = L["Right"],
						LEFT = L["Left"],
					},
				},
				alpha = {
					type = "range",
					order = 30,
					disabled = function()
						return not MBI.db.profile.ignite.power.enabled
					end,
					name = L["Alpha"],
					min = 0.2, max = 1, step = 0.05,
				},
				space2 = {
					type = "description",
					name = " ",
					width = "full",
					order = 40,
				},
				gradient = {
					type = "select",
					disabled = function()
						return not MBI.db.profile.ignite.power.enabled
					end,
					name = L["Gradient colour"],
					order = 50,
					values = {
						rg = L["Red to green"],
						bg = L["Blue to green"],
					},
				},
				middle = {
					type = "range",
					order = 60,
					disabled = function()
						return not MBI.db.profile.ignite.power.enabled
					end,
					name = L["Gradient middle"],
					desc = L["Percentage of the power bar at which is placed the midle of the colour gradient."],
					isPercent = true,
					min = 0.5, max = 0.8, step = 0.05,
				},				
				space3 = {
					type = "description",
					name = " ",
					width = "full",
					order = 70,
				},
				tableSize = {
					type = "range",
					order = 80,
					disabled = function()
						return not MBI.db.profile.ignite.power.enabled
					end,
					name = L["Number of ticks"],
					desc = L["Set the number of last ticks to consider in calculations. Higher value means more stable but less reactive power bar."],
					min = 50, max = 150, step = 10,
				},
				reset = {
					type = "execute",
					name = L["Reset numbers"],
					desc = L["Reset the numbers driving the power bar."],
					order = 90,
					confirm = true,
					confirmText = L["Sure?"],
					func = function()
						wipe(MBI.db.profile.stats.power)
						MBI:InitPowerBar()
					end,
				},
				space = {
					type = "description",
					name = " ",
					width = "full",
					order = 500,
				},
			},
		},
		layout = {
			type = "group",
			inline = true,
			order = 90,
			args = {
				header = {
					type = "header",
					name =  L["Ignite frame layout"],
					order = 0,
				},
				text = {
					type = "group",
					inline = true,
					order = 10,
					args = {
						align = {
							type = "select",
							name = L["Text alignement"],
							order = 0,
							values = {
								RIGHT = L["Right"],
								LEFT = L["Left"],
								CENTER = L["Center"],
							},
						},
						ofx = {
							type = "range",
							name = L["Horizontal offset"],
							order = 10,
							min = -10, max = 10, step = 0.05, bigStep = 0.25,
						},
						ofy = {
							type = "range",
							name = L["Vertical offset"],
							order = 20,
							min = -2.5, max = 2.5, step = 0.05, bigStep = 0.25,
						},
						format = {
							type = "select",
							name = L["Text format"],
							order = 40,
							values = {
								["normal"] = L["Normal"],
								["condensed"] = L["Condensed"],
							},
						},
						space = {
							type = "description",
							name = " ",
							width = "full",
							order = 500,
						},
					},
				},
				background = {
					type = "group",
					inline = true,
					order = 20,
					args = {
						enabled = {
							type = "toggle",
							name = L["Enable background"],
							desc = L["Enables main window background."],
							order = 0,
						},
						darkness = {
							type = "range",
							name = L["Background brightness"],
							order = 10,
							disabled = function()
								return not MBI.db.profile.ignite.layout.background.enabled
							end,
							min = 0, max = 1, step = 0.01,
							isPercent = true,
						},
						inset = {
							type = "range",
							name = L["Background inset"],
							order = 20,
							disabled = function()
								return not MBI.db.profile.ignite.layout.background.enabled
							end,
							min = -10, max = 10, step = 0.05,
						},
						space = {
							type = "description",
							name = " ",
							width = "full",
							order = 500,
						},
					},
				},
				border = {
					type = "group",
					inline = true,
					order = 30,
					args = {
						enabled = {
							type = "toggle",
							name = L["Enable border"],
							desc = L["Enables main window border."],
							order = 0,
						},
						size = {
							type = "range",
							name = L["Border thickness"],
							order = 10,
							disabled = function()
								return not MBI.db.profile.ignite.layout.border.enabled
							end,
							min = 0, max = 35, step = 0.05 ,
						},
						space = {
							type = "description",
							name = " ",
							width = "full",
							order = 500,
						},
					},
				},
				adjust = {
					type = "range",
					name = L["Edge spacing"],
					desc = L["Adjusts the space between the inside frame (timer, power and display) and the border or the edge of the background."],
					order = 40,
					min = 0, max = 5, step = 0.05 ,
				},
				space = {
					type = "description",
					name = " ",
					width = "full",
					order = 500,
				},
			},
		},
	}

	return options
end



--##############################################################################################################################################
-- managing of config panel --------------------------------------------------------------------------------------------------------------------

-- open config function
function MBI:OpenConfig()
	MBI.db.RegisterCallback(MBI, "OnProfileChanged", "RefreshConfig")
	MBI.db.RegisterCallback(MBI, "OnProfileCopied", "RefreshConfig")
	MBI.db.RegisterCallback(MBI, "OnProfileReset", "RefreshConfig")
	InterfaceOptionsFrame_OpenToCategory("myBigIgnite")
end

-- switch between real and fake options
function MBI:ChangeConfigDisplay(state)
	wipe(MBI.options.args)
	if state == "ooFireSpec" then
		if MBI.ooFOptions == nil then
			MBI:LoadooFOptions()
		end
		MBI.options.args = DeepCopy(MBI.ooFOptions.args)
	elseif state == "fireSpec" then
		if MBI.fireOptions == nil then
			MBI:LoadFireOptions()
		end
		MBI.options.args = DeepCopy(MBI.fireOptions.args)
		MBI.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(MBI.db)
	else
		MBI.options.args = DeepCopy(MBI.fakeOptions.args)
	end
	ACR:NotifyChange("myBigIgnite")
end

-- profiles changing
function MBI:RefreshConfig()
	MBI:CreateSpellTables()
	MBI:InitStatModule()
	MBI:SetIgniteFrame()
	MBI:UpdateIgniteFrame("low")
	MBI:UpdateIgniteTimerDisplay("low", 6)
	MBI:UpdateIgnitePowerDisplay(0)
	if MBI.db.profile.threshold.value == 0 then
		print(MBI.ORANGE .. L["myBigIgnite: "] .. MBI.YELLOW_LIGHT .. L["notyetthreshold"])
	end
	if MBI.db.profile.general.autohide.noCombat then
		MBI.F.main:Hide()
	else
		MBI.F.main:Show()
	end
end

-- resets layout
function MBI:ResetLayout()
	MBI.font = LSM:Fetch("font", MBI.db.profile.general.layout.font)
	MBI:SetIgniteFrame()
	if not MBI.cfMode and MBI.db.profile.general.autohide.noCombat then
		MBI.F.main:Hide()
	end
end



--##############################################################################################################################################
-- config mode associated functions ------------------------------------------------------------------------------------------------------------

-- config mode
function MBI:ConfigMode(cfmode)
	if cfmode then
		MBI.F.anchor:SetMovable(true)
		MBI.F.main:EnableMouse(true)
		MBI.F.main:Show()
		MBI.F.main:SetScript("OnMouseDown", function(self, button)
			MBI.F.anchor.isMoving = true
			MBI.F.anchor:StartMoving()
		end)
		MBI.F.main:SetScript("OnMouseUp", function(self, button)
				if MBI.F.anchor.isMoving then
					local UIScale = UIParent:GetScale()
					MBI.F.anchor.isMoving = nil
					MBI.F.anchor:StopMovingOrSizing()
					MBI.db.profile.general.anchor.coord.xcd, MBI.db.profile.general.anchor.coord.ycd = MBI.F.anchor:GetLeft() * UIScale, MBI.F.anchor:GetBottom() * UIScale
				end
		end)

		MBI.F.config = MBI.F.config or CreateFrame("Frame", nil, MBI.F.anchor)
		MBI.F.config:SetMovable(false)
		MBI.F.config:EnableMouse(false)
		MBI.F.config:Show()
		local i = 0
		local igRemain = 6
		MBI.F.config:SetScript("OnUpdate", function(self, elapsed)
			igRemain = igRemain - elapsed
			if igRemain < 0 then
				igRemain = 6
			end
			self.TimeSinceLastUpdate = (self.TimeSinceLastUpdate or 0) + elapsed
			if MBI.inCombat then
				MBI.cfMode = false
				MBI:ConfigMode(MBI.cfMode)
			end
			if i == 0 then
				MBI:UpdateIgniteFrame("low")
				MBI:UpdateIgniteTimerDisplay("low", igRemain)
				MBI.F.timer.txt:SetText(FormatText(0))
				if MBI.db.profile.ignite.timer.enabled then
					MBI.F.timer:SetValue(igRemain)
				else
					MBI.F.timer:SetValue(0)
				end
				if MBI.db.profile.ignite.power.enabled then
					MBI.F.power.endValue = 0.3
				end
			elseif i == 1 then
				if MBI.db.profile.threshold.interm.enabled then
					MBI:UpdateIgniteFrame("mid")
					MBI:UpdateIgniteTimerDisplay("mid", igRemain)
					MBI.F.timer.txt:SetText(FormatText(MBI.db.profile.threshold.value * MBI.db.profile.threshold.interm.perct))
				end
				if MBI.db.profile.ignite.timer.enabled then
					MBI.F.timer:SetValue(igRemain)
				else
					MBI.F.timer:SetValue(0)
				end
				if MBI.db.profile.ignite.power.enabled then
					MBI.F.power.endValue = MBI.db.profile.ignite.power.average
				end
			elseif i == 2 then
				MBI:UpdateIgniteFrame("high")
				MBI:UpdateIgniteTimerDisplay("high", igRemain)
				MBI.F.timer.txt:SetText(FormatText(MBI.db.profile.threshold.value))
				if MBI.db.profile.ignite.timer.enabled then
					MBI.F.timer:SetValue(igRemain)
				else
					MBI.F.timer:SetValue(0)
				end
				if MBI.db.profile.ignite.power.enabled then
					MBI.F.power.endValue = 1
				end
			end
			if (self.TimeSinceLastUpdate > MBI.REFRESH_CM) then
				i = i + 1
				if i > 2 then
					i = 0
				end
				self.TimeSinceLastUpdate = 0
			end
		end)
	else
		MBI.F.timer.txt:SetText(FormatText(0))
		MBI.F.anchor:SetMovable(false)
		MBI.F.main:EnableMouse(false)
		MBI.F.config:Hide()
		MBI.F.timer:SetValue(0)
		MBI.F.power.endValue = 0
		MBI:UpdateIgniteFrame("low")
		if MBI.db.profile.general.autohide.noCombat then
			MBI.F.main:Hide()
		end
	end
end









--------------------------------- OLD SHIT ---------------------------------------------


					--icone = {
					--	type = "group",
					--	inline = true,
					--	order = 95,
					--	args = {
					--		iconHeader = {
					--			type = "header",
					--			name = L["Icons"],
					--			order = 0,
					--		},
					--		isIcon = {
					--			type = "toggle",
					--			name = L["Combustion CD icon"],
					--			desc = L["Enables display of dynamic icon."],
					--			order = 5,
					--			get = function()
					--				return MBI.db.profile.icons.enabled
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.icons.enabled = value
					--				if MBI.db.profile.icons.enabled and not MBI.F.icon then
					--					MBI:CreateIconFrame()
					--				end
					--				if MBI.db.profile.icons.enabled then
					--					MBI:SetFrameSize("icon")
					--					MBI:SetFramePoint("icon", MBI.db.profile.icons.side, MBI.F.main, MBI.db.profile.icons.side, MBI.F.icon.off, 0)
					--					MBI.F.icon:Show()
					--				else
					--					MBI.F.icon:Hide()
					--				end
					--				MBI:SetFrameSize("icon")
					--				MBI:SetFrameSize("main")
					--				MBI:SetFramePoint("icon", MBI.db.profile.icons.side, MBI.F.main, MBI.db.profile.icons.side, MBI.F.icon.off, 0)
					--				MBI:SetFramePoint("dmg", MBI.db.profile.ignite.layout.text.align, MBI.F.main, MBI.db.profile.ignite.layout.text.align, 0, 0)
					--				MBI:AlignText("dmg")
					--			end,
					--		},
					--		space4 = {
					--			type = "description",
					--			name = " ",
					--			widh = "full",
					--			order = 10,
					--		},
					--		space5 = {
					--			type = "description",
					--			name = " ",
					--			widh = "full",
					--			order = 15,
					--		},
					--		iconeSize = {
					--			type = "range",
					--			name = L["Icon size"],
					--			order = 20,
					--			disabled = function()
					--				return not MBI.db.profile.icons.enabled
					--			end,
					--			isPercent = true,
					--			min = 0.5,
					--			max = 1.5,
					--			softMin = 0.6,
					--			softMax = 1.45,
					--			step = 0.01,
					--			bigStep = 0.05,
					--			get = function()
					--				return MBI.db.profile.icons.sizeFactor
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.icons.sizeFactor = value
					--				MBI:SetFrameSize("icon")
					--				MBI:SetFrameSize("main")
					--				MBI:SetFramePoint("icon", MBI.db.profile.icons.side, MBI.F.main, MBI.db.profile.icons.side, MBI.F.icon.off, 0)
					--			end,
					--		},
					--		iconSide = {
					--			type = "select",
					--			name = L["Icon side"],
					--			order = 25,
					--			disabled = function()
					--				return not MBI.db.profile.icons.enabled
					--			end,
					--			values = {
					--				RIGHT = L["Right"],
					--				LEFT = L["Left"],
					--			},
					--			get = function()
					--				return MBI.db.profile.icons.side
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.icons.side = value
					--				MBI:SetFramePoint("icon", MBI.db.profile.icons.side, MBI.F.main, MBI.db.profile.icons.side, MBI.F.icon.off, 0)
					--				MBI:SetFramePoint("dmg", MBI.db.profile.ignite.layout.text.align, MBI.F.main, MBI.db.profile.ignite.layout.text.align, 0, 0)
					--				MBI:AlignText("dmg")
					--			end,
					--		},
					--		iconeType = {
					--			type = "select",
					--			name = L["Icon type"],
					--			desc = L["Changes Ignite icon skin."],
					--			order = 30,
					--			disabled = function()
					--				return not MBI.db.profile.icons.enabled
					--			end,
					--			values = {
					--				normal = L["Normal"],
					--				zoomed = L["Zoomed"],
					--				normbord = L["Normal with border"],
					--				zoombord = L["Zoomed with border"],
					--			},
					--			get = function()
					--				return MBI.db.profile.icons.type
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.icons.type = value
					--				MBI:SetIconType("icon", MBI.db.profile.icons.type)
					--				MBI:SetFrameBackdrop("icon")
					--				MBI:SetIconBorderColor(MBI.db.profile.threshold.low.bd)
					--			end,
					--		},
					--		bdIconThickness = {
					--			type = "range",
					--			name = L["Icon border thickness"],
					--			order = 35,
					--			disabled = isIcone,
					--			min = 0.2,
					--			max = 5,
					--			softMin = 0.25,
					--			softMax = 5,
					--			step = 0.1,
					--			bigStep = 0.25,
					--			get = function()
					--				return MBI.db.profile.icons.bdSize
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.icons.bdSize = value
					--				MBI:SetFrameBackdrop("icon")
					--				MBI:SetIconBorderColor(MBI.db.profile.threshold.low.bd)
					--			end,
					--		},
					--	},
					--},






					--livingBomb = {
					--	type = "toggle",
					--	name = L["livingBomb"],
					--	desc = L["Toggle to activate Living Bomb dot tracking."],
					--	order = 2,
					--	width = "full",
					--	get = function ()
					--		return MBI.db.profile.spellTracking.livingBomb.enabled
					--	end,
					--	set = function (info, value)
					--		MBI.db.profile.spellTracking.livingBomb.enabled = value
					--		MBI:CreateSpellTables()
					--	end,
					--},




					--space2 = {
					--	type = "description",
					--	name = " ",
					--	width = "full",
					--	order = 8,
					--},
					--optionalDesc = {
					--	type = "description",
					--	name = "|cffFFC000" .. L["Enable optional frames:"],
					--	fontSize = "medium",
					--	width = "full",
					--	order = 9,
					--},
					--isIcon = {
					--	type = "toggle",
					--	name = L["Icons"],
					--	desc = L["Enables display of the Ignite icon."],
					--	order = 10,
					--	get = function()
					--		return MBI.db.profile.icons.enabled
					--	end,
					--	set = function(info, value)
					--		MBI.db.profile.icons.enabled = value
					--		if MBI.db.profile.icons.enabled and not MBI.F.icon then
					--			MBI:CreateIconFrame()
					--		end
					--		if MBI.db.profile.icons.enabled then
					--			MBI:SetFrameSize("icon")
					--			MBI:SetFramePoint("icon", MBI.db.profile.icons.side, MBI.F.main, MBI.db.profile.icons.side, MBI.F.icon.off, 0)
					--			MBI.F.icon:Show()
					--		else
					--			MBI.F.icon:Hide()
					--			MBI.db.profile.icons.impactIcon = false
					--			if not MBI.db.profile.values.impactPerctEnabled then
					--				MBI.db.profile.ignite.impactMode = false
					--			end
					--		end
					--		MBI:SetFrameSize("icon")
					--		MBI:SetFrameSize("main")
					--		MBI:SetFramePoint("icon", MBI.db.profile.icons.side, MBI.F.main, MBI.db.profile.icons.side, MBI.F.icon.off, 0)
					--		MBI:SetFramePoint("dmg", MBI.db.profile.ignite.layout.text.align, MBI.F.main, MBI.db.profile.ignite.layout.text.align, 0, 0)
					--		MBI:AlignText("dmg")
					--		if MBI.db.profile.titleFrame.enabled then
					--			MBI:SetFramePoint("title", MBI.db.profile.ignite.layout.text.align, MBI.F.main, MBI.db.profile.ignite.layout.text.align, 0, 0)
					--			MBI:AlignText("title")
					--		end
					--	end,
					--},
					--isTitle = {
					--	type = "toggle",
					--	name = L["Title"],
					--	desc = L["Enables display of \"Ignite\" on the top of the frame."],
					--	order = 11,
					--	get = function()
					--		return MBI.db.profile.titleFrame.enabled
					--	end,
					--	set = function(info, value)
					--		MBI.db.profile.titleFrame.enabled = value
					--		if MBI.db.profile.titleFrame.enabled and not MBI.F.title then
					--			MBI:CreateTitleFrame()
					--		end
					--		if MBI.db.profile.titleFrame.enabled then
					--			MBI:SetFrameSize("title")
					--			MBI:SetFramePoint("title", MBI.db.profile.ignite.layout.text.align, MBI.F.main, MBI.db.profile.ignite.layout.text.align, 0, 0)
					--			MBI.F.title:Show()
					--		else
					--			MBI.F.title:Hide()
					--		end
					--		if MBI.db.profile.icons.enabled then
					--			MBI:SetFrameSize("icon")
					--		end
					--		MBI:SetFrameSize("main")
					--		MBI:UpdateDisplay("low")
					--	end,
					--},
					-- isHUD = {
						-- type = "toggle",
						-- name = L["HUD"],
						-- desc = L["Enables Power Aura style notification."],
						-- order = 10,
						-- disabled = true,
						-- get = function()
							-- return MBI.db.profile.hudFrame.enabled
						-- end,
						-- set = function(info, value)
							-- MBI.db.profile.hudFrame.enabled = value
						-- end,
					-- },




--function MBI:TemplateOptions()																		-- loop to add template options
--	local args = {}
--	local tempType = {
--		[1] = "default",
--		[2] = "original",
--		[3] = "modern",
--		[4] = "classic",
--		[5] = "ancient",
--		[6] = "simple",
--	}
--	for i = 1, #tempType do
--		args[tempType[i]] = {
--			type = "execute",
--			name = L[tempType[i]],
--			desc = L["Applies template: "] .. L[tempType[i]],
--			image = "Interface\\Addons\\myBigIgnite\\buttons\\" .. tempType[i] .. ".tga",
--			imageWidth = 128,
--			imageHeight = 64,
--			order = i,
--			confirm = true,
--			confirmText = L["This will change the current layout settings. Are you sure?"],
--			func = function()
--				MBI:ApplyTemplate(tempType[i])
--			end,
--		}
--	end
--
--	wipe(MBI.fireOptions.args.tabTemplates.args)
--	MBI.fireOptions.args.tabTemplates.args = DeepCopy(args)
--end


			-- tabHUD = {
				-- type = "group",
				-- name = L["HUD"],
				-- desc = L["Heads up display settings."],
				-- order = 7,
				-- disabled = true,
				-- args = {},
			-- },









					--imp_combuHeader = {
					--	type = "header",
					--	name = L["Combustion and Impact tracking"],
					--	order = 10,
					--},
					--combuTracking = {
					--	type = "toggle",
					--	name = L["Combustion cooldown"],
					--	order = 11,
					--	desc = L["Toggle to activate Combustion cooldown tracking."],
					--	width = "full",
					--	get = function()
					--		return MBI.db.profile.ignite.combuTrack
					--	end,
					--	set = function(info, value)
					--		MBI.db.profile.ignite.combuTrack = value
					--		if not value then
					--			MBI.db.profile.ignite.impactMode = false
					--			MBI.combuCD = false
					--			MBI.combuCDTimer = 0
					--		else
					--			MBI:CombuCheck()
					--		end
					--	end,
					--},
					--impactTracking= {
					--	type = "toggle",
					--	name = L["Impact mode"],
					--	order = 12,
					--	desc = L["Toggle to activate Impact mode."],
					--	disabled = function()
					--		return not MBI.db.profile.ignite.combuTrack
					--	end,
					--	get = function()
					--		return MBI.db.profile.ignite.impactMode
					--	end,
					--	set = function(info, value)
					--		MBI.db.profile.ignite.impactMode = value
					--		MBI.db.profile.values.impactPerctEnabled = value
					--		if MBI.db.profile.icons.enabled then
					--			MBI.db.profile.icons.impactIcon = value
					--		end
					--	end,
					--},
					--impactPerctEnable = {
					--	type = "toggle",
					--	name = L["Threshold"],
					--	desc = L["Enables threshold for Impact notification."],
					--	order = 13,
					--	disabled = function()
					--		return not MBI.db.profile.ignite.impactMode
					--	end,
					--	get = function()
					--		return MBI.db.profile.values.impactPerctEnabled
					--	end,
					--	set = function(info, value)
					--		MBI.db.profile.values.impactPerctEnabled = value
					--		if not value and not MBI.db.profile.icons.impactIcon then
					--			MBI.db.profile.ignite.impactMode = value
					--		end
					--	end,
					--},
					--impactIconEnable = {
					--	type = "toggle",
					--	name = L["Icons"],
					--	desc = L["Enables icon for Impact notification."],
					--	order = 14,
					--	disabled = function()
					--		return not MBI.db.profile.ignite.impactMode or not MBI.db.profile.icons.enabled
					--	end,
					--	get = function()
					--		return MBI.db.profile.icons.impactIcon
					--	end,
					--	set = function(info, value)
					--		MBI.db.profile.icons.impactIcon = value
					--		if not value and not MBI.db.profile.values.impactPerctEnabled then
					--			MBI.db.profile.ignite.impactMode = value
					--		end
					--	end,
					--},
					--priorityHeader = {
					--	type = "header",
					--	name = L["Informative icon"],
					--	order = 20,
					--},
					--space1 = {
					--	type = "description",
					--	name = " ",
					--	width = "full",
					--	order = 30,
					--},
					--prioFiller = {
					--	type = "select",
					--	name = L["Your filler spell"],
					--	order = 31,
					--	disabled = function()
					--		return not MBI.db.profile.icons.enabled
					--	end,
					--	values = {
					--		["fb"] = L["Fireball"],
					--		["ffb"] = L["ffbolt"],
					--	},
					--	get = function()
					--		return MBI.db.profile.icons.filler
					--	end,
					--	set = function(info, value)
					--		MBI.db.profile.icons.filler = value
					--	end,
					--},
			--	},
			--},
			--tabReac = {
			--	type = "group",
			--	name = L["Emphasis"],
			--	desc = L["Sets how display will be adjusted upon Ignite tick value."],
			--	order = 6,
			--	args = {},
			--},
			----tabAdvanced = {
			--	type = "group",
			--	name = L["Advanced"],
			--	desc = L["Advanced stuff, be carefull!"],
			--	order = 8,
			--	args = {
					--header1 = {
					--	type = "header",
					--	order = 1,
					--	name = L["Red Zone"],
					--},
					--redZone = {
					--	type = "range",
					--	name = L["Red Zone"],
					--	desc = L["Time before expiration of one of the DoTs that will put myBigIgnite display in the low Ignite tick state."],
					--	order = 2,
					--	min = 0.1,
					--	max = 1,
					--	softMin = 0.25,
					--	softMax = 0.75,
					--	step = 0.05,
					--	bigStep = 0.05,
					--	get = function()
					--		return MBI.db.profile.ignite.lag
					--	end,
					--	set = function(info, value)
					--		MBI.db.profile.ignite.lag = value
					--	end,
					--},
					--redZoneVisual = {
					--	type = "toggle",
					--	name = L["Visual red zone"],
					--	desc = L["Display will be of the choosen colour under the red zone time."],
					--	order = 3,
					--	get = function()
					--		return MBI.db.profile.ignite.lagVisual
					--	end,
					--	set = function(info, value)
					--		MBI.db.profile.ignite.lagVisual = value
					--	end,
					--},
					--redZoneColor = {
					--	type = "color",
					--	name = L["Red zone colour"],
					--	order = 4,
					--	disabled = function()
					--		return not MBI.db.profile.ignite.lagVisual
					--	end,
					--	get = function()
					--		return GetColor(MBI.db.profile.ignite.lagColor)
					--	end,
					--	set = function(info, r, g ,b)
					--		SetColor(MBI.db.profile.ignite.lagColor, r, g, b)
					--	end,
					--},
					--redZoneType = {
					--	type = "multiselect",
					--	name = L["Red zone type"],
					--	desc= L["Selects how red zone is displayed."],
					--	order = 5,
					--	disabled = function()
					--		return not MBI.db.profile.ignite.lagVisual
					--	end,
					--	values = {
					--		text = L["Text"],
					--		bg = L["Background"],
					--		bd = L["Border"],
					--	},
					--	get = function(info, key)
					--		return MBI.db.profile.ignite.lagVisualType[key]
					--	end,
					--	set = function(info, key, value)
					--		MBI.db.profile.ignite.lagVisualType[key] = value
					--	end,
					--},
					--space1 = {
					--	type = "description",
					--	name = " ",
					--	width = "full",
					--	order = 6,
					--},
					--igniteTimer = {
					--	type = "group",
					--	inline = true,
					--	order = 7,
					--	args = {
					--		header1 = {
					--			type = "header",
					--			order = 1,
					--			name = L["Ignite timer"],
					--		},
					--		enabled = {
					--			type = "toggle",
					--			order = 3,
					--			name = L["Enable"],
					--			desc = L["Activates a timer bar showing visually the remaining time of an Ignite dot."],
					--			get = function()
					--				return MBI.db.profile.ignite.timer.enabled
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.ignite.timer.enabled = value
					--				if MBI.db.profile.ignite.timer.enabled then
					--					if not MBI.F.timer then
					--						MBI.F.timer = CreateFrame("StatusBar", nil, MBI.F.main)
					--					end
					--					MBI:SetIgniteTimer(MBI.F.timer)
					--				end
					--				if value then
					--					MBI.F.timer:Show()
					--				else
					--					MBI.F.timer:Hide()
					--				end
					--			end,
					--		},
					--		space1 = {
					--			type = "description",
					--			name = " ",
					--			width = "full",
					--			order = 5,
					--		},
					--		reverse = {
					--			type = "toggle",
					--			order = 7,
					--			disabled = function()
					--				return not MBI.db.profile.ignite.timer.enabled
					--			end,
					--			name = L["Reversed"],
					--			get = function()
					--				return MBI.db.profile.ignite.timer.reverse
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.ignite.timer.reverse = value
					--				MBI:SetIgniteTimer(MBI.F.timer)
					--			end,
					--		},
					--		useGeneralBackground = {
					--			type = "toggle",
					--			order = 9,
					--			disabled = function()
					--				return not MBI.db.profile.ignite.timer.enabled
					--			end,
					--			name = L["Background texture"],
					--			desc = L["myBigIgnite background texture is applied to the bar."],
					--			get = function()
					--				return MBI.db.profile.ignite.timer.useGeneralBackground
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.ignite.timer.useGeneralBackground = value
					--				MBI:SetIgniteTimer(MBI.F.timer)
					--			end,
					--		},
					--		texture = {
					--			type = "select",
					--			order = 11,
					--			disabled = function()
					--				return not MBI.db.profile.ignite.timer.enabled or MBI.db.profile.ignite.timer.useGeneralBackground
					--			end,
					--			name = L["Texture"],
					--			values = List("statusbar"),
					--			get = function()
					--				return MBI.db.profile.general.layout.bar
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.general.layout.bar = value
					--				MBI:SetIgniteTimer(MBI.F.timer)
					--			end,
					--		},
					--		space2 = {
					--			type = "description",
					--			name = " ",
					--			width = "full",
					--			order = 12,
					--		},
					--		sizeMult = {
					--			type = "range",
					--			order = 13,
					--			disabled = function()
					--				return not MBI.db.profile.ignite.timer.enabled
					--			end,
					--			name = L["Size multiplier"],
					--			desc = L["Controls the size of the bar."],
					--			isPercent = true,
					--			min = 0.5, max = 2, bigStep = 0.05,
					--			get = function()
					--				return MBI.db.profile.ignite.timer.mult
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.ignite.timer.mult = value
					--				MBI:SetIgniteTimer(MBI.F.timer)
					--			end,
					--		},
					--		ofx = {
					--			type = "range",
					--			order = 14,
					--			disabled = function()
					--				return not MBI.db.profile.ignite.timer.enabled
					--			end,
					--			name = L["Horizontal offset"],
					--			min = -20, max = 20, bigStep = 0.05,
					--			get = function()
					--				return MBI.db.profile.ignite.timer.ofx
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.ignite.timer.ofx = value
					--				MBI:SetIgniteTimer(MBI.F.timer)
					--			end,
					--		},
					--		ofy = {
					--			type = "range",
					--			order = 15,
					--			disabled = function()
					--				return not MBI.db.profile.ignite.timer.enabled
					--			end,
					--			name = L["Vertical offset"],
					--			min = -20, max = 20, bigStep = 0.05,
					--			get = function()
					--				return MBI.db.profile.ignite.timer.ofy
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.ignite.timer.ofy = value
					--				MBI:SetIgniteTimer(MBI.F.timer)
					--			end,
					--		},
					--		space3 = {
					--			type = "description",
					--			name = " ",
					--			width = "full",
					--			order = 16,
					--		},
					--		alpha = {
					--			type = "range",
					--			order = 17,
					--			disabled = function()
					--				return not MBI.db.profile.ignite.timer.enabled
					--			end,
					--			name = L["Alpha"],
					--			min = 0, max = 1, bigStep = 0.05,
					--			get = function()
					--				return MBI.db.profile.ignite.timer.alpha
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.ignite.timer.alpha = value
					--			end,
					--		},
					--		hasSpeColor = {
					--			type = "toggle",
					--			order = 18,
					--			disabled = function()
					--				return not MBI.db.profile.ignite.timer.enabled
					--			end,
					--			name = L["Toggle colour"],
					--			desc = L["Allow the bar to has its own colour."],
					--			get = function()
					--				return MBI.db.profile.ignite.timer.specificColor.enabled
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.ignite.timer.specificColor.enabled = value
					--			end,
					--		},
					--		speColor = {
					--			type = "color",
					--			order = 19,
					--			disabled = function()
					--				return not MBI.db.profile.ignite.timer.enabled or not MBI.db.profile.ignite.timer.specificColor.enabled
					--			end,
					--			name = L["Specific colour"],
					--			get = function()
					--				return GetColor(MBI.db.profile.ignite.timer.specificColor.color)
					--			end,
					--			set = function(info, r, g, b)
					--				SetColor(MBI.db.profile.ignite.timer.specificColor.color, r, g, b)
					--			end,
					--		},
					--		space4 = {
					--			type = "description",
					--			name = " ",
					--			width = "full",
					--			order = 20,
					--		},
					--		redZone = {
					--			type = "range",
					--			order = 21,
					--			disabled = function()
					--				return not MBI.db.profile.ignite.timer.enabled
					--			end,
					--			name = L["Ignite red zone"],
					--			desc = L["The timer bar will turn of the selected colour when the remaining time of Ignite is under this value."],
					--			min = 0, max = 2, step = 0.05, bigStep = 0.1,
					--			get = function()
					--				return MBI.db.profile.ignite.timer.redZone
					--			end,
					--			set = function(info, value)
					--				MBI.db.profile.ignite.timer.redZone = value
					--			end,
					--		},
					--		rzColor = {
					--			type = "color",
					--			order = 23,
					--			disabled = function()
					--				return not MBI.db.profile.ignite.timer.enabled
					--			end,
					--			name = L["Red zone colour"],
					--			get = function()
					--				return GetColor(MBI.db.profile.ignite.timer.redZoneColor)
					--			end,
					--			set = function(info, r, g, b)
					--				SetColor(MBI.db.profile.ignite.timer.redZoneColor, r, g, b)
					--			end,
					--		},
					--		space5 = {
					--			type = "description",
					--			name = " ",
					--			width = "full",
					--			order = 25,
					--		},
					--	},
					--},
					--header2 = {
					--	type = "header",
					--	order = 8,
					--	name = L["Advanced settings"],
					--},
					--refresh = {
					--	type = "range",
					--	name = L["Refresh"],
					--	desc = L["Better to know what you're doing..."],
					--	order = 9,
					--	min = 0.1,
					--	max = 1,
					--	softMin = 0.1,
					--	softMax = 1,
					--	step = 0.05,
					--	bigStep = 0.05,
					--	get = function()
					--		return MBI.REFRESH
					--	end,
					--	set = function(info, value)
					--		MBI.REFRESH = value
					--	end,
					--},
					--refreshCM = {
					--	type = "range",
					--	name = L["C.M. refresh"],
					--	desc = L["Config mode refresh speed."],
					--	order = 10,
					--	min = 0.5,
					--	max = 3,
					--	softMin = 0.5,
					--	softMax = 2,
					--	step = 0.1,
					--	bigStep = 0.1,
					--	get = function()
					--		return MBI.REFRESH_CM
					--	end,
					--	set = function(info, value)
					--		MBI.REFRESH_CM = value
					--	end,
					--},
					--space2 = {
					--	type = "description",
					--	name = " ",
					--	width = "full",
					--	order = 1,
					--},
			--		splashMsg = {
			--			type = "toggle",
			--			name = L["Splash message"],
			--			desc = L["Enables the welcome message on myBigIgnite loading."],
			--			order = 12,
			--			get = function()
			--				return MBI.db.profile.general.splashMsg
			--			end,
			--			set = function(info, value)
			--				MBI.db.profile.general.splashMsg = value
			--			end,
			--		},
			--	},
			--},
	--	},
	--}

	--MBI:WarningOptions()
	--MBI:PriorityOptions()
--end

--function MBI:PriorityOptions()
--	local args = {}
--	local index = {
--		[1] = "livingBomb",
--		[2] = "ffbolt",
--		[3] = "pyroblast",
--		[4] = "ignite",
--	}

	--for i = 1, #index do
	--	args["prio" .. index[i]] = {
	--		type = "select",
	--		name = L[index[i]],
	--		order = 24 + i,
	--		disabled = function()
	--			return not MBI.db.profile.icons.enabled or not MBI.db.profile.spellTracking[index[i]].enabled or not MBI.db.profile.icons.
	--		end,
	--		values = function()
	--			local t = {}
	--				for j = 1, #index do
	--					t[j] = j
	--				end
	--			return t
	--		end,
	--		get = function()
	--			return MBI.db.profile.spellTracking[index[i]].order
	--		end,
	--		set = function(info, value)
	--			local tempOrder = MBI.db.profile.spellTracking[index[i]].order
	--			MBI.db.profile.spellTracking[index[i]].order = value
	--
	--			for k, v in pairs(MBI.db.profile.spellTracking) do
	--				if k ~= index[i] then
	--					if MBI.db.profile.spellTracking[k].order == value then
	--						MBI.db.profile.spellTracking[k].order = tempOrder
	--					end
	--				end
	--			end
	--		end,
	--	}
	--	tinsert(MBI.fireOptions.args.tabSpells.args, args["prio" .. index[i]])
	--end
--end



--function MBI:ApplyTemplate(template)																-- switches database settings
--	MBI:Templates()
--	wipe(MBI.db.profile.ignite)
--	wipe(MBI.db.profile.icons)
--	wipe(MBI.db.profile.titleFrame)
--	MBI.db.profile.ignite = selectCopy(MBI.templates[template].mainFrame, MBI.defaults.profile.ignite)
--	MBI.db.profile.icons = selectCopy(MBI.templates[template].iconFrame, MBI.defaults.profile.icons)
--	MBI.db.profile.titleFrame = selectCopy(MBI.templates[template].titleFrame, MBI.defaults.profile.titleFrame)
--
--	MBI:ResetLayout()																						-- applies settings
--end
