﻿--[[
Title: myBigIgnite
Author: Istaran | Medivh - EU
Version: 3.1.0
Release date: 2012-11-20T20:16:10Z
myBigIgnite is licensed under GPLv3.
]]



--##############################################################################################################################################
-- ace addon creation and initialization -------------------------------------------------------------------------------------------------------

-- checks if mBI must be enabled at all
if select(2, UnitClass("player")) ~= "MAGE" then
   return
end

myBigIgnite = LibStub("AceAddon-3.0"):NewAddon("myBigIgnite", "AceConsole-3.0", "AceEvent-3.0")
local MBI = myBigIgnite

-- libraries initialization
local L = LibStub("AceLocale-3.0"):GetLocale("myBigIgnite", true)
local LSM = LibStub("LibSharedMedia-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")

-- cache
local pairs, ipairs, type, tonumber, tostring, print, tinsert, tremove, wipe = pairs, ipairs, type, tonumber, tostring, print, tinsert, tremove, wipe
local strfind, strsub, floor, ceil, format = strfind, strsub, floor, ceil, format
local _

-- addon tables, contants and defaults
do
	--[===[@debug@
	MBI.VERSION = "3.1.0"
	--@end-debug@]===]
	MBI.VERSION = MBI.VERSION or GetAddOnMetadata("myBigIgnite", "Version")

	MBI.ORANGE = "|cffFF8000"
	MBI.YELLOW_LIGHT = "|cffFFE680"
	MBI.YELLOW = "|cffFFD101"
	MBI.NORMAL = "|cffFFFFFF"

	MBI.REFRESH = 0.1
	MBI.REFRESH_CM = 1

	MBI.F = {}
	MBI.remainTime = {}
	MBI.dotToTrack = {}
	MBI.dotIcon = {}
	MBI.dotName = {}
	MBI.dotOrder = {}
	
	MBI.spellID = {
		ignite = 12654,
		-- livingBomb = 44457,
		pyroblast = 11366,
		combustion = 11129,
		fireball = 133,
		inferno = 108853,
		masteryIgnite = 12846,
	}

	MBI.defaults = {
		profile = {
			general = {
				anchor = {
					coord = {
						xcd = UIParent:GetWidth() / 2,
						ycd = UIParent:GetHeight() / 2,
					},
					strata = 2,
					align = "RIGHT",
				},
				autohide = {
					noCombat = false,
					noIgnite = false,
					combuCD = false,
					combuCDTime = 0,
				},
				layout = {
					textSize = 12,
					font = "Friz Quadrata TT",
					background = "Solid",
					border = "Solid",
					bar = "BantoBar",
					alpha = 1,
				},
				splashMsg = true,
				resetStats = 1,
			},
			ignite = {
				layout = {
					background = {
						enabled = true,
						inset = 0.75,
						darkness = 0.1
					},
					border = {
						enabled = true,
						size = 0.75,
					},
					text = {
						format = "condensed",
						align = "RIGHT",
						ofx = 0.75,
						ofy = 1.25,
					},
				adjust = 1.5,
				},
				timer = {
					enabled = true,
					reverse = false,
					redZone = 1,
					alpha = 1,
					tick = true,
					redZoneColor = {
						r = 1,
						g = 0.2,
						b = 0,
					},
				},
				power = {
					enabled = true,
					align = "LEFT",
					alpha = 1,
					gradient = "rg",
					middle = 0.7,
					tableSize = 100,
				},
				dmgAsDps = false,
				useCombatLog = false,
				smoothDmgReport = true,
				smoothDmgValue = 0.6,
			},
			threshold = {
				value = 0,
				interm = {
					enabled = true,
					perct = 0.85,
				},
				low = {
					color = {
						r = 1,
						g = 0.5,
						b = 0,
					},
				},
				mid = {
					color = {
						r = 1,
						g = 1,
						b = 0,
					},
					mult = 1.2,
				},
				high = {
					color = {
						r = 0,
						g = 1,
						b = 0,
					},
					mult = 1.4,
				},
			},
			trackpyro = true,
		},
	}
end



--##############################################################################################################################################
-- housekeeping functions ----------------------------------------------------------------------------------------------------------------------

local function GetColor(t)
	return t.r, t.g, t.b, t.a
end

local function DeepCopy(src)
	local res = {}
    for k, v in pairs(src) do
        if type(v) == "table" then
            v = DeepCopy(v)
        end
        res[k] = v
    end
    return res
end

local function Round(number, dec)
	return math.floor(number * 10^dec + 0.5) / 10^dec
end

local function FormatText(txt)
	if type(txt) == "number" then
		if MBI.db.profile.ignite.layout.text.format == "condensed" then
			txt = Round(txt / 1000, 1) .. "k"
		end
	end
	return txt
end

local function HlString(str, colorL, colorR)
	local colorL = colorL or MBI.ORANGE
	local colorR = colorR or MBI.NORMAL
	return colorL .. str .. colorR
end



--##############################################################################################################################################
-- loading process -----------------------------------------------------------------------------------------------------------------------------

-- on enable
function MBI:OnEnable()
	MBI:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED", "Load")
	MBI:RegisterEvent("PLAYER_LEVEL_UP", "Load")
	MBI:RegisterEvent("PLAYER_ENTERING_WORLD", "Load")
	MBI:LoadFakeOptions()
end

-- loads/unloads addon elements
function MBI:Load()
	-- loads components if player in fire spec, do nothing if already in
	if IsSpellKnown(MBI.spellID.masteryIgnite) and not MBI.specFire then
		local showHelp
		-- addon loading first time
		if not MBI.addonLoadded then
			-- erase user db when updating from version < 3.1.x
			if _G["mBIsettings"] and (not _G["mBIsettings"].global or not _G["mBIsettings"].global.version or _G["mBIsettings"].global.version < "3.1.0") then
				wipe(_G["mBIsettings"])
				MBI.temp = true
			end

			-- db and cmd
			MBI.db = LibStub("AceDB-3.0"):New("mBIsettings", MBI.defaults)
			MBI:RegisterChatCommand("mybigignite","CmdParser")
			MBI:RegisterChatCommand("mbi", "CmdParser")

			-- stats and power bar
			MBI:InitStatModule()
			MBI:InitPowerBar()
			
			-- update addon version stored in db if necessary
			if not MBI.db.global.version or MBI.db.global.version < MBI.VERSION then
				if not MBI.db.global.version or string.match(MBI.db.global.version, "(%d+.%d+).%d+") < string.match(MBI.VERSION, "(%d+.%d+).%d+") then
					showHelp = true
				end
				MBI.db.global.version = MBI.VERSION
			end

			--shared media stuff
			LSM:Register("border", "Solid", "Interface\\Buttons\\WHITE8X8")

			local texturesPath = "Interface\\Addons\\myBigIgnite\\bars\\"
			LSM:Register("statusbar", "Aluminum", texturesPath .. "Aluminum.tga")
			LSM:Register("statusbar", "Armory", texturesPath .. "Armory.tga")
			LSM:Register("statusbar", "BantoBar", texturesPath .. "BantoBar.tga")
			LSM:Register("statusbar", "Charcoal", texturesPath .. "Charcoal.tga")
			LSM:Register("statusbar", "Flat", texturesPath .. "Flat.tga")
			LSM:Register("statusbar", "Frost", texturesPath .. "Frost.tga")
			LSM:Register("statusbar", "Glaze", texturesPath .. "Glaze.tga")
			LSM:Register("statusbar", "Gloss", texturesPath .. "Gloss.tga")
			LSM:Register("statusbar", "Graphite", texturesPath .. "Graphite.tga")
			LSM:Register("statusbar", "Healbot", texturesPath .. "Healbot.tga")
			LSM:Register("statusbar", "LiteStep", texturesPath .. "LiteStep.tga")
			LSM:Register("statusbar", "Minimalist", texturesPath .. "Minimalist.tga")
			LSM:Register("statusbar", "Otravi", texturesPath .. "Otravi.tga")
			LSM:Register("statusbar", "Perl", texturesPath .. "Perl.tga")
			LSM:Register("statusbar", "Rocks", texturesPath .. "Rocks.tga")
			LSM:Register("statusbar", "Runes", texturesPath .. "Runes.tga")
			LSM:Register("statusbar", "Shard", texturesPath .. "Shard.tga")
			LSM:Register("statusbar", "Smooth", texturesPath .. "Smooth.tga")
			LSM:Register("statusbar", "Striped", texturesPath .. "Striped.tga")
			LSM:Register("statusbar", "Xeon", texturesPath .. "Xeon.tga")

			-- addon font
			MBI.font = LSM:Fetch("font", MBI.db.profile.general.layout.font)

			-- event registring
			MBI:RegisterEvent("PLAYER_REGEN_DISABLED", "CombatCheck")
			MBI:RegisterEvent("PLAYER_REGEN_ENABLED", "CombatCheck")
			MBI:RegisterEvent("UI_SCALE_CHANGED", "ScaleChange")

			-- creates general tables storing settings + frames
			MBI:CreateSpellTables()
			MBI:SetIgniteFrame()
			MBI:CreateOnUpdateFrame()
			
			MBI.addonLoadded = true

		-- back into active state
		else
			MBI:RegisterEvent("PLAYER_REGEN_DISABLED", "CombatCheck")
			MBI:RegisterEvent("PLAYER_REGEN_ENABLED", "CombatCheck")
			MBI:RegisterEvent("UI_SCALE_CHANGED", "ScaleChange")
			MBI:RegisterChatCommand("mybigignite","CmdParser")
			MBI:RegisterChatCommand("mbi", "CmdParser")
			MBI.F.anchor:Show()
		end

		-- timers init
		MBI.time_before_stop_onupdate = 10
		MBI.smoothDmg = 0
		
		-- combustion CD checking
		MBI:CombuCheck()

		-- config panel switches
		if MBI.configLoaded then
			MBI:ChangeConfigDisplay("fireSpec")
		else
			MBI:ChangeConfigDisplay("unloadedConfig")
		end

		-- splash message
		if MBI.db.profile.general.splashMsg then
			print(MBI.ORANGE .. L["myBigIgnite v"] .. MBI.VERSION .. L[" loaded."])
		end

		--warning if threshold unset
		if MBI.db.profile.threshold.value == 0 then
			print(MBI.ORANGE .. L["myBigIgnite: "] .. MBI.YELLOW_LIGHT .. L["notyetthreshold"])
		end

		-- hides MBI frame if required
		if MBI.db.profile.general.autohide.noCombat then
			MBI.F.main:Hide()
		else
			MBI.F.main:Show()
		end
		
		-- show help if necessary
		if showHelp then
			MBI:OpenInfo("help")
		end

		MBI.specFire = true

	-- unload when ignite unkown and was in fire spec
	elseif not IsSpellKnown(MBI.spellID.masteryIgnite) and MBI.specFire then
		MBI:UnregisterEvent("PLAYER_REGEN_DISABLED")
		MBI:UnregisterEvent("PLAYER_REGEN_ENABLED")
		MBI:UnregisterEvent("UI_SCALE_CHANGED")
		MBI:UnregisterChatCommand("mybigignite")
		MBI:UnregisterChatCommand("mbi")
		MBI.F.anchor:Hide()

		-- splash message
		if MBI.db.profile.general.splashMsg then
			print(MBI.ORANGE .. L["myBigIgnite is disabled."])
		end

		-- non fire spec config panel state
		MBI:ChangeConfigDisplay("ooFireSpec")

		MBI.specFire = false

	-- load non fire spec config pannel if not in fire spec and wasn't previously
	elseif not IsSpellKnown(MBI.spellID.masteryIgnite) and not MBI.specFire then
		MBI:ChangeConfigDisplay("ooFireSpec")
	end
end

-- stat module init
function MBI:InitStatModule()
	MBI.db.profile.stats = MBI.db.profile.stats or {}
	MBI.db.profile.stats.info = MBI.db.profile.stats.info or {}
	MBI.db.profile.stats.info.average = MBI.db.profile.stats.info.average or 0
	MBI.db.profile.stats.info.dmg = MBI.db.profile.stats.info.dmg or 0
	MBI.db.profile.stats.info.count = MBI.db.profile.stats.info.count or 0
	MBI.db.profile.stats.info.highest = MBI.db.profile.stats.info.highest or 0
	MBI.db.profile.stats.info.lowest = MBI.db.profile.stats.info.lowest or 100000
	MBI.db.profile.stats.info.supOptT = MBI.db.profile.stats.info.supOptT or 0
	MBI.db.profile.stats.info.supIntermT = MBI.db.profile.stats.info.supIntermT or 0
	MBI.db.profile.stats.info.thresholdCount = MBI.db.profile.stats.info.thresholdCount or 0
end

-- power bar init
function MBI:InitPowerBar()
	MBI.db.profile.stats.power = MBI.db.profile.stats.power or {}
	MBI.power = {}
	
	-- check integrity of value table + init of average and sigma value
	if #MBI.db.profile.stats.power > 0 then
		while #MBI.db.profile.stats.power > MBI.db.profile.ignite.power.tableSize do
			tremove(MBI.db.profile.stats.power, 1)
		end
		MBI:CalculatePower()
	else
		MBI.power.average = 0
		MBI.power.sigma = 0
	end
end

----- OLD STUFF THAT MUST GO AWAY ONCE 5.1 IS LIVE -------
-- internal spell tables created from user's database
function MBI:CreateSpellTables()
	wipe(MBI.dotToTrack)
	wipe(MBI.dotIcon)
	wipe(MBI.dotName)
	wipe(MBI.dotOrder)

	local dotOrder = {
		ignite = 0,
		pyroblast = 1,
	}
	local spellToTrack = {
		ignite = true
	}
	if MBI.db.profile.trackpyro then
		spellToTrack.pyroblast = true
	else
		spellToTrack.pyroblast = nil
	end
	
	local i = 1
	for k in pairs(spellToTrack) do
		MBI.dotToTrack[i], _, MBI.dotIcon[i] = GetSpellInfo(MBI.spellID[k])
		MBI.dotName[k] = MBI.dotToTrack[i]
		MBI.dotOrder[i] = dotOrder[k]
		i = i + 1
	end
end
-----------------------------------------------------------------



--##############################################################################################################################################
-- frame creation / setup ----------------------------------------------------------------------------------------------------------------------

-- ignite frame setting
function MBI:SetIgniteFrame()
	-- create frames
	MBI.F.anchor = MBI.F.anchor or CreateFrame("Frame", nil, UIParent)
	MBI.F.main = MBI.F.main or CreateFrame("Frame", nil, MBI.F.anchor)
	MBI.F.power = MBI.F.power or CreateFrame("StatusBar", nil, MBI.F.main)
	MBI.F.timer = MBI.F.timer or CreateFrame("StatusBar", nil, MBI.F.main)
	MBI.F.tick = MBI.F.tick or {}
	for i = 0, 2 do
		MBI.F.tick[i] = MBI.F.tick[i] or CreateFrame("Frame", nil, MBI.F.timer)
	end

	-- set the anchor
	do
		MBI.F.anchor:SetWidth(1)
		MBI.F.anchor:SetHeight(1)
		MBI.F.anchor:SetMovable(false)
		MBI.F.anchor:EnableMouse(false)

		MBI.F.anchor:ClearAllPoints()
		local ofx = MBI.db.profile.general.anchor.coord.xcd / UIParent:GetScale() - UIParent:GetWidth() / 2
		local ofy = MBI.db.profile.general.anchor.coord.ycd / UIParent:GetScale() - UIParent:GetHeight() / 2
		MBI.F.anchor:SetPoint("CENTER", UIParent, "CENTER", ofx , ofy)

		if MBI.db.profile.general.anchor.strata == 1 then
			MBI.F.anchor:SetFrameStrata("LOW")
		elseif MBI.db.profile.general.anchor.strata == 2 then
			MBI.F.anchor:SetFrameStrata("MEDIUM")
		else
			MBI.F.anchor:SetFrameStrata("HIGH")
		end
	end

	-- ignite timer and dmg display
	do
		-- timer and dmg
		do
			MBI.F.timer.txt = MBI.F.timer.txt or MBI.F.timer:CreateFontString()
			MBI.F.timer.txt:SetFont(MBI.font, MBI.db.profile.general.layout.textSize)
			MBI.F.timer.txt:SetShadowColor(0, 0, 0, 1)
			MBI.F.timer.txt:SetShadowOffset(1, -1)

			local ofx = MBI.db.profile.ignite.layout.text.ofx
			local ofy = MBI.db.profile.ignite.layout.text.ofy
			MBI.F.timer.txt:ClearAllPoints()
			if point == "RIGHT" then
				ofx = -ofx
			end
			MBI.F.timer.txt:SetPoint(MBI.db.profile.ignite.layout.text.align, MBI.F.timer, MBI.db.profile.ignite.layout.text.align, ofx, ofy)

			MBI.F.timer.txt:SetText(FormatText(999999))
			local width = MBI.F.timer.txt:GetStringWidth() * 1.2
			local height = MBI.F.timer.txt:GetStringHeight() * 1.4
			MBI.F.timer.w = width
			MBI.F.timer.h = height
			MBI.F.timer.txt:SetText(FormatText(0))

			MBI.F.timer:SetStatusBarTexture(LSM:Fetch("statusbar", MBI.db.profile.general.layout.bar))
			MBI.F.timer:SetReverseFill(MBI.db.profile.ignite.timer.reverse)
			MBI.F.timer:SetMovable(false)
			MBI.F.timer:EnableMouse(false)

			MBI.F.timer:ClearAllPoints()
			local adjust = MBI.db.profile.ignite.layout.adjust
			if MBI.db.profile.ignite.power.enabled and MBI.db.profile.ignite.power.align == "LEFT" then
				MBI.F.timer:SetPoint("TOPRIGHT", MBI.F.main, "TOPRIGHT", -adjust, -adjust)
				MBI.F.timer:SetPoint("BOTTOMRIGHT", MBI.F.main, "BOTTOMRIGHT", -adjust, adjust)
			elseif MBI.db.profile.ignite.power.enabled and MBI.db.profile.ignite.power.align == "RIGHT" then
				MBI.F.timer:SetPoint("TOPLEFT", MBI.F.main, "TOPLEFT", adjust, -adjust)
				MBI.F.timer:SetPoint("BOTTOMLEFT", MBI.F.main, "BOTTOMLEFT", -adjust, adjust)
			else
				MBI.F.timer:SetPoint("TOPLEFT", MBI.F.main, "TOPLEFT", adjust, -adjust)
				MBI.F.timer:SetPoint("BOTTOMRIGHT", MBI.F.main, "BOTTOMRIGHT", -adjust, adjust)
			end

			MBI.F.timer:SetMinMaxValues(0, 6)
			MBI.F.timer:SetValue(0)

			MBI.F.timer:SetScript("OnUpdate", function(self, elapsed)
				if MBI.F.timer.value and MBI.F.timer.value > 0 then
					MBI.F.timer.value = MBI.F.timer.value - elapsed
					MBI.F.timer:SetValue(MBI.F.timer.value)
				end
			end)

			if MBI.cfMode then
				MBI.F.timer:SetValue(6)
			end
		end

		-- ticks marks
		do
			local size = 0.4 * MBI.F.timer.h
			MBI.F.tick.size = size
			for i = 0, 2 do
				MBI.F.tick[i]:SetMovable(false)
				MBI.F.tick[i]:EnableMouse(false)
				if MBI.db.profile.ignite.timer.reverse then
					MBI.F.tick[i]:SetBackdrop({bgFile = [[Interface\Addons\myBigIgnite\textures\tick_reverse.tga]]})
				else
					MBI.F.tick[i]:SetBackdrop({bgFile = [[Interface\Addons\myBigIgnite\textures\tick.tga]]})
				end
				if MBI.db.profile.ignite.timer.enabled and MBI.db.profile.ignite.timer.tick then
					MBI.F.tick[i]:Show()
				else
					MBI.F.tick[i]:Hide()
				end
			end
		end
	end

	-- power bar
	do
		MBI.F.power:SetStatusBarTexture(LSM:Fetch("statusbar", MBI.db.profile.general.layout.bar))
		MBI.F.power:SetOrientation("VERTICAL")
		MBI.F.power:SetRotatesTexture(true)
		MBI.F.power:SetWidth(1)
		MBI.F.power:SetHeight(1)
		MBI.F.power:SetMovable(false)
		MBI.F.power:EnableMouse(false)

		MBI.F.power:ClearAllPoints()

		if MBI.db.profile.ignite.power.align == "LEFT" then
			MBI.F.power:SetPoint("TOPLEFT", MBI.F.main, "TOPLEFT", MBI.db.profile.ignite.layout.adjust, -MBI.db.profile.ignite.layout.adjust)
			MBI.F.power:SetPoint("BOTTOMRIGHT", MBI.F.timer, "BOTTOMLEFT", -1, 0)
		else
			MBI.F.power:SetPoint("TOPRIGHT", MBI.F.main, "TOPRIGHT", -MBI.db.profile.ignite.layout.adjust, -MBI.db.profile.ignite.layout.adjust)
			MBI.F.power:SetPoint("BOTTOMLEFT", MBI.F.timer, "BOTTOMRIGHT", 1, 0)
		end

		MBI.F.power:SetMinMaxValues(0, 1)
		MBI.F.power:SetValue(0)

		MBI.F.power:SetScript("OnUpdate", function(self, elapsed)
			if MBI.F.power.endValue then
				MBI.F.power.value = MBI.F.power.value or 0
				if MBI.F.power.value > MBI.F.power.endValue then
					MBI.F.power.value = MBI.F.power.value - elapsed
					if MBI.F.power.value <= MBI.F.power.endValue then
						MBI.F.power.value = MBI.F.power.endValue
					end
					MBI.F.power:SetValue(MBI.F.power.value)
					MBI:UpdateIgnitePowerDisplay(MBI.F.power.value)
				elseif MBI.F.power.value < MBI.F.power.endValue then
					MBI.F.power.value = MBI.F.power.value + elapsed
					if MBI.F.power.value >= MBI.F.power.endValue then
						MBI.F.power.value = MBI.F.power.endValue
					end
					MBI.F.power:SetValue(MBI.F.power.value)
					MBI:UpdateIgnitePowerDisplay(MBI.F.power.value)
				end
			end
		end)

		if MBI.db.profile.ignite.power.enabled then
			MBI.F.power:Show()
		else
			MBI.F.power:Hide()
		end
	end
	
	-- main frame
	do
		MBI.F.main:ClearAllPoints()
		MBI.F.main:SetPoint(MBI.db.profile.general.anchor.align, MBI.F.anchor, "CENTER")
		MBI.F.main:SetMovable(false)
		MBI.F.main:EnableMouse(false)
		MBI.F.main:SetClampedToScreen(true)
		MBI.F.main:SetAlpha(MBI.db.profile.general.layout.alpha)

		local backdrop = {
			bgFile = LSM:Fetch("background", MBI.db.profile.general.layout.background),
			edgeFile = LSM:Fetch("border", MBI.db.profile.general.layout.border),
			edgeSize = MBI.db.profile.ignite.layout.border.size,
			insets = {
				left = MBI.db.profile.ignite.layout.background.inset,
				right = MBI.db.profile.ignite.layout.background.inset,
				top = MBI.db.profile.ignite.layout.background.inset,
				bottom = MBI.db.profile.ignite.layout.background.inset,
			},
		}
		if not MBI.db.profile.ignite.layout.background.enabled then
			backdrop.bgFile = ""
		end
		if not MBI.db.profile.ignite.layout.border.enabled then
			backdrop.edgeFile = ""
		end
		MBI.F.main:SetBackdrop(backdrop)

		local border = MBI.db.profile.ignite.layout.adjust
		local width
		if MBI.db.profile.ignite.power.enabled then
			width = MBI.F.timer.w * 1.25 + border * 2
		else
			width = MBI.F.timer.w + border * 2
		end
		local height = MBI.F.timer.h + border * 2
		MBI.F.main.w = width
		MBI.F.main.h = height
		MBI.F.main:SetWidth(width)
		MBI.F.main:SetHeight(height)
	end

	MBI:UpdateIgniteFrame("low")
	MBI:UpdateIgniteTimerDisplay("low", 6)
	MBI:UpdateIgnitePowerDisplay(0)
end

-- on update frame
function MBI:CreateOnUpdateFrame()
	MBI.F.combat = CreateFrame("Frame", nil, MBI.F.anchor)
	MBI.F.combat:SetMovable(false)
	MBI.F.combat:EnableMouse(false)
	MBI.F.combat:SetPoint("CENTER", MBI.F.anchor, "CENTER")
	MBI.F.combat:SetScript("OnUpdate",  function(self, elapsed) MBI.MainEngine(self, elapsed) end)
	MBI.F.combat:Hide()
end



--##############################################################################################################################################
-- info and stats frame functions --------------------------------------------------------------------------------------------------------------

-- info and stats helper function
function MBI:OpenInfo(mode, isCmd)
	if not MBI.F.info then
		MBI:SetInfoFrame()
		MBI.infoTable = {}
	end
	
	MBI:PopulateInfo(mode)
	MBI:InfoScrollUpdate()
	MBI.F.info:Show()
	
	if isCmd then
		MBI.F.info.shownFromCmd = true
	else
		InterfaceOptionsFrame:Hide()
		HideUIPanel(GameMenuFrame)
	end
end

-- info frame
function MBI:SetInfoFrame()
	local R, G, B = 1, 0.5, 0
	local infoW, infoH = 700, 600
	
	-- info frame
	local info = myBigIgnite_info
	info:SetWidth(infoW)
	info:SetHeight(infoH)
	info:SetPoint("TOP", UIParent, "TOP", 0, -100)
	info:SetBackdrop({bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", edgeFile = "Interface\\Buttons\\WHITE8X8", edgeSize = 2})
	info:SetBackdropBorderColor(R, G, B)
	info:EnableMouse(true)
	info:SetMovable(true)

	info:SetScript("OnMouseDown", function()
		info:StartMoving()
	end)
	info:SetScript("OnMouseUp", function()
		info:StopMovingOrSizing()
	end)
	
	-- scroll
	local scroll = myBigIgnite_infoScroll
	scroll:SetPoint("TOPLEFT", info, "TOPLEFT", 2, -75)
	scroll:SetPoint("BOTTOMRIGHT", info, "BOTTOMRIGHT", -30, 50)
	info.scroll = scroll
	
	-- title
	local title = CreateFrame("Frame", nil, info)
	title:SetHeight(25)
	title:SetPoint("TOP", info, "TOP", 0, -10)
	title:SetBackdrop({bgFile = "", edgeFile = "Interface\\Buttons\\WHITE8X8", edgeSize = 0.75})
	title:SetBackdropBorderColor(R, G, B)
	title.txt = title:CreateFontString()
	title.txt:SetPoint("CENTER", title, "CENTER")
	title.txt:SetFont(MBI.font, 12)
	title.txt:SetShadowColor(0, 0, 0, 1)
	title.txt:SetShadowOffset(1, -1)
	info.title = title
	
	-- close button	
	local close = MBI:MakeButton(L["Close"], info)
	close:SetPoint("BOTTOMRIGHT", info, "BOTTOMRIGHT", -30, 15)
	close:SetScript("OnClick", function()
		info:Hide()
		if not MBI.F.info.shownFromCmd then
			InterfaceOptionsFrame_OpenToCategory("myBigIgnite")
		end
		MBI.F.info.shownFromCmd = false
	end)

	-- lines
	local maxLine = 26
	local lineHeight = 18
	local lines = {}
	for i = 1, maxLine do
		local line = lines[i]
		line = scroll:CreateFontString()
		line:SetPoint("TOPLEFT", scroll, "TOPLEFT", 7, -lineHeight * (i - 1))
		line:SetWidth(infoW - 40)
		line:SetShadowColor(0, 0, 0, 1)
		line:SetShadowOffset(1, -1)
		line:SetWordWrap(true)
		line:SetSpacing(4)
		lines[i] = line
	end
	info.lines = lines
	
	MBI.F.info = info
	MBI.F.info:Hide()
end

-- button factory
function MBI:MakeButton(text, parent)
	local R, G, B = 1, 0.5, 0
	
	local btn = CreateFrame("Button", nil, parent)
	btn:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8X8", edgeFile = "Interface\\Buttons\\WHITE8X8", edgeSize = 0.75})
	btn:SetBackdropColor(R * 0.3, G * 0.3, B * 0.3)
	btn:SetBackdropBorderColor(R, G, B)
	btn.txt = btn:CreateFontString()
	btn.txt:SetPoint("CENTER", btn, "CENTER")
	btn.txt:SetFont("Fonts\\FRIZQT__.TTF", 13)
	btn.txt:SetShadowColor(0, 0, 0, 1)
	btn.txt:SetShadowOffset(1, -1)
	btn.txt:SetTextColor(R, G, B)
	btn.txt:SetText(text)
	btn:SetWidth(btn.txt:GetStringWidth() * 1.4)
	btn:SetHeight(btn.txt:GetStringHeight() * 1.3)
	btn.highL = btn:CreateTexture(nil, "HIGHLIGHT")
	btn.highL:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	btn.highL:SetAllPoints(btn)
	btn.highL:SetBlendMode("ADD")
	btn.highL:SetAlpha(0.5)
	btn:SetScript("OnMouseDown", function()
		btn.txt:ClearAllPoints()
		btn.txt:SetPoint("CENTER", btn, "CENTER", 1, -1)
	end)
	btn:SetScript("OnMouseUp", function()
		btn.txt:ClearAllPoints()
		btn.txt:SetPoint("CENTER", btn, "CENTER")
	end)
	
	return btn
end

-- info scroll frame update
function MBI:InfoScrollUpdate()
	local maxLine = 26
	local lineHeight = 18
	local lineOffset, nLine
	
	local i, j = 1, 1
	while i <= maxLine do
		MBI.F.info.lines[i]:Hide()
		if MBI.infoTable[i] and MBI.infoTable[i].type == "button" then
			MBI.infoTable[i].value:ClearAllPoints()
			MBI.infoTable[i].value:Hide()
		end
		i = i + 1
	end
	i = 1
	while i <= maxLine do
		lineOffset = j + FauxScrollFrame_GetOffset(myBigIgnite_infoScroll)
		if lineOffset <= #MBI.infoTable then
			if MBI.infoTable[lineOffset].type == "text" or string.find(MBI.infoTable[lineOffset].type, "title") or MBI.infoTable[lineOffset].type == "special" then
				local fontSize = 11
				MBI.F.info.lines[i]:SetJustifyH("LEFT")
				if MBI.infoTable[lineOffset].type == "title1" then
					fontSize = 15
				elseif MBI.infoTable[lineOffset].type == "title2" then
					fontSize = 12
				elseif MBI.infoTable[lineOffset].type == "special" then
					fontSize = 15
					MBI.F.info.lines[i]:SetJustifyH("CENTER")
				end
				
				MBI.F.info.lines[i]:SetFont("Fonts\\FRIZQT__.TTF", fontSize)
				MBI.F.info.lines[i]:SetText(MBI.infoTable[lineOffset].value)
				nLine = math.ceil(MBI.F.info.lines[i]:GetStringHeight() / lineHeight)
				MBI.infoTable[lineOffset].line = nLine
				MBI.F.info.lines[i]:Show()
				if i + nLine > maxLine then
					MBI.F.info.lines[i]:Hide()
				end
				i = i + nLine - 1
			elseif MBI.infoTable[lineOffset].type == "button" then
				MBI.F.info.lines[i]:Hide()
				local btn = MBI.infoTable[lineOffset].value
				btn:ClearAllPoints()
				btn:SetPoint("TOPLEFT", MBI.F.info.scroll, "TOPLEFT", 7, -lineHeight * (i - 1))
				btn:Show()
				local nLine = math.ceil(btn:GetHeight() / lineHeight)
				MBI.infoTable[lineOffset].line = nLine
				if i + nLine > maxLine then
					btn:ClearAllPoints()
				end
				i = i + nLine - 1
			end
		else
			MBI.F.info.lines[i]:Hide()
		end
		i = i + 1
		j = j + 1
	end
	
	local realLineNum = maxLine + 1
	local tot = 0
	for entry in ipairs(MBI.infoTable) do
		tot = tot + MBI.infoTable[entry].line
	end
	if tot > realLineNum then
		realLineNum = tot
	end
	
	FauxScrollFrame_Update(myBigIgnite_infoScroll, realLineNum, maxLine, lineHeight)
end

-- info table update
function MBI:PopulateInfo(mode)
	for i = 1, #MBI.infoTable do
		if MBI.infoTable[i].type == "button" then
			MBI.infoTable[i].value:ClearAllPoints()
			MBI.infoTable[i].value:Hide()
		end
	end
	wipe(MBI.infoTable)
	local info = MBI.infoTable
	
	if mode == "stats" then
		MBI.F.info.title.txt:SetText(format(HlString(L["myBigIgnite: "]) .. L["statistics"]))
		MBI.F.info.title:SetWidth(MBI.F.info.title.txt:GetStringWidth() * 1.1)
		
		-- ignite damages
		table.insert(info, {
			type = "title1",
			value = MBI.ORANGE .. L["Ignite statistics"],
		})
		table.insert(info, {
			type = "text",
			value = format(L["    total damages: %s"], MBI.ORANGE .. MBI.db.profile.stats.info.dmg),
		})
		table.insert(info, {
			type = "text",
			value = format(L["    total ticks: %s"], MBI.ORANGE .. MBI.db.profile.stats.info.count),
		})
		table.insert(info, {
			type = "text",
			value = format(L["    average tick damages: %s"], MBI.ORANGE .. Round(MBI.db.profile.stats.info.average, 1)),
		})
		local lowest = MBI.db.profile.stats.info.lowest
		if MBI.db.profile.stats.info.lowest == 100000 then
			lowest = 0
		end
		table.insert(info, {
			type = "text",
			value = format(L["    lowest tick damages: %s"], MBI.ORANGE .. lowest),
		})
		table.insert(info, {
			type = "text",
			value = format(L["    highest tick damages: %s"], MBI.ORANGE .. MBI.db.profile.stats.info.highest),
		})
		table.insert(info, {
			type = "text",
			value = " ",
		})
		local count = MBI.db.profile.stats.info.thresholdCount
		if MBI.db.profile.stats.info.thresholdCount == 0 then	-- workaround to avoid div by 0
			count = 1
		end
		table.insert(info, {
			type = "text",
			value = format(L["    number of ticks since last change of threshold: %s"], MBI.ORANGE .. MBI.db.profile.stats.info.thresholdCount),
		})
		table.insert(info, {
			type = "text",
			value = format(L["    percentage of ticks above optimal threshold: %s"], MBI.ORANGE .. Round(MBI.db.profile.stats.info.supOptT / count, 4) * 100 .. "%"),
		})
		if MBI.db.profile.threshold.interm.enabled then
			table.insert(info, {
				type = "text",
				value = format(L["    percentage of ticks between intermediate and optimal threshold: %s"], MBI.ORANGE .. Round(MBI.db.profile.stats.info.supIntermT / count, 4) * 100 .. "%"),
			})
		end
		table.insert(info, {
			type = "text",
			value = " ",
		})

		-- combu damages
		
		-- reset button
		table.insert(info, {
			type = "text",
			value = " ",
		})
		table.insert(info, {
			type = "text",
			value = " ",
		})
		MBI.F.info.statReset = MBI.F.info.statReset or MBI:MakeButton(L["Reset all stats"], MBI.F.info.scroll)
		MBI.F.info.statReset:SetScript("OnClick", function()
			wipe(info)
			wipe(MBI.db.profile.stats.info)
			MBI:InitStatModule()
			MBI:PopulateInfo("stats")
			MBI:InfoScrollUpdate()
		end)
		table.insert(info, {
			type = "button",
			value = MBI.F.info.statReset,
		})
		
		info.type = "stats"
	else
		MBI.F.info.title.txt:SetText(format(HlString(L["myBigIgnite: "]) .. L["help"]))
		MBI.F.info.title:SetWidth(MBI.F.info.title.txt:GetStringWidth() * 1.1)
		
		-- version and author
		table.insert(info, {
			type = "text",
			value = format("version %s     Istaran | Medivh - EU", MBI.ORANGE .. MBI.VERSION),
		})
		table.insert(info, {
			type = "text",
			value = " ",
		})
		
		-- reset for 3.1.0
		if MBI.temp then
			table.insert(info, {
				type = "special",
				value = MBI.ORANGE .. "Because of the HUGE number of changes behind the scene, I had to reset entirely the settings again. Sorry for the inconvenience, hopping this is the last time.",
			})
			table.insert(info, {
				type = "text",
				value = " ",
			})
			table.insert(info, {
				type = "text",
				value = " ",
			})
			MBI.temp = false
		end
		
		-- no threshold
		if MBI.db.profile.threshold.value == 0 then
			table.insert(info, {
				type = "special",
				value = format(MBI.ORANGE .. "Your threshold is not set yet.\nGo to Threshold tab in configuration window or type %s to set it.", HlString("/mbi threshold [value]", MBI.NORMAL, MBI.ORANGE)),
			})
			table.insert(info, {
				type = "text",
				value = " ",
			})
			table.insert(info, {
				type = "text",
				value = " ",
			})
		end
		
		-- news
		table.insert(info, {
			type = "title1",
			value = MBI.ORANGE .. "Latest additions",
		})
		table.insert(info, {
			type = "text",
			value = format([[
%s a bar that shows the amount of the current Ignite tick on a normalized scale.
%s a scale over the ignite timer that shows when Ignite ticks happen.
%s you can now choose to turn it off by setting it to 1.
%s the general layout has been cleaned and polished.
%s the config UI has been entirely reworked and greatly simplified.
%s statistics about Ignite damages are stored. You can access them by typing %s or using the button in the config UI.
%s general informations about addon usage are now available in game. You can access them by typing %s or using the button in the config UI.
			]], HlString("- ignite power bar:", MBI.YELLOW), HlString("- ignite tick marks:", MBI.YELLOW), HlString("- addon magnification:", MBI.YELLOW), HlString("- new layout:", MBI.YELLOW), HlString("- new configuration options:", MBI.YELLOW), HlString("- statistics:", MBI.YELLOW), HlString("/mbi stats"), HlString("- help:", MBI.YELLOW), HlString("/mbi help")),
		})
		table.insert(info, {
			type = "text",
			value = " ",
		})
		
		-- features
		table.insert(info, {
			type = "title1",
			value = MBI.ORANGE .. "Features",
		})
		table.insert(info, {
			type = "title2",
			value = MBI.YELLOW .. "Visual warning",
		})
		table.insert(info, {
			type = "text",
			value = "myBigIgnite provides a visual warning when the current Ignite tick damage is above your threshold. The availability of Combustion is also taken into account. Thus, if Ignite tick damage is sufficient and Combustion available, the display is updated: color and size are adapted to the value of Ignite. ",
		})
		table.insert(info, {
			type = "title2",
			value = MBI.YELLOW .. "Damage display",
		})
		table.insert(info, {
			type = "text",
			value = "myBigIgnite shows current Ignite tick damage in a very clear way. Advanced options can change the way damages are displayed; they can be set up in Ignite frame tab.",
		})
		table.insert(info, {
			type = "title2",
			value = MBI.YELLOW .. "Ignite timer",
		})
		table.insert(info, {
			type = "text",
			value = "A timer representing the remaining time of Ignite is provided with marks that show when Ignite ticks happen. The timer has a configurable red zone to better know if using Combustion is risky or not.",
		})
		table.insert(info, {
			type = "title2",
			value = MBI.YELLOW .. "Ignite power bar",
		})
		table.insert(info, {
			type = "text",
			value = "Ignite power bar in another way to represent how high is the amount of damages done by the current Ignite tick. This feature is fully automatized and is here to help you to visualize the current Ignite tick together with the threshold.\nBriefly, the average damage and deviation are calculated over a given amount of ticks. The middle of the bar is set to the average value of Ignite ticks and the maximum of the bar correponds to 3 times the deviation. These numbers are adjusted regularly as you are playing. Some advanced options can be changed in the configuration.",
		})
		table.insert(info, {
			type = "title2",
			value = MBI.YELLOW .. "Statistics",
		})
		table.insert(info, {
			type = "text",
			value = "Statistics about the damages done by Ignite and Combustion(NYI) are stored and can be accessed. This is for number lovers and also can be used to properly adjust your threshold.",
		})
		table.insert(info, {
			type = "text",
			value = " ",
		})
		
		-- usage
		table.insert(info, {
			type = "title1",
			value = MBI.ORANGE .. "Usage",
		})
		table.insert(info, {
			type = "title2",
			value = MBI.YELLOW .. "Accessing the config",
		})
		table.insert(info, {
			type = "text",
			value = format("Type %s or use the game menu to acces config options. Enabling %s will make the frame movable, animate the frame and shows it if necessary.", HlString("/mbi config"), HlString("Config mode")), 
		})
		table.insert(info, {
			type = "title2",
			value = MBI.YELLOW .. "Setting the threshold",
		})
		table.insert(info, {
			type = "text",
			value = format("To set up your threshold, go to Threshold tab in configuration window. You can also quickly set up your optimal threshold by typing %s.\n A good threshold must allow you to capture rare events. You can use the stats registered by myBigIgnite as well as the the power bar to help you setiing properly your threshold.", HlString("/mbi threshold [value]")),
		})
		table.insert(info, {
			type = "title2",
			value = MBI.YELLOW .. "Showing stats and help",
		})
		table.insert(info, {
			type = "text",
			value = format("Type %s to show several statistics about Ignite and Combustion or use the button in the config options. Type %s to show this help.", HlString("/mbi stats"), HlString("/mbi help")),
		})
		table.insert(info, {
			type = "text",
			value = " ",
		})
		table.insert(info, {
			type = "text",
			value = MBI.ORANGE .. "Suggestions and bug reports can be made on Curse site [http://www.curse.com/addons/wow/mybigignite] either in comments or by pm.",
		})
		
		info.type = "help"
	end
	
	for entry in ipairs(info) do
		info[entry].line = 1
	end
	
	MBI.infoTable = info
end



--##############################################################################################################################################
-- live layout update functions ---------------------------------------------------------------------------------------------------------------

-- updates ignite frame size and color
function MBI:UpdateIgniteFrame(igtype)
	local mult = MBI.db.profile.threshold[igtype].mult or 1
	local txtSize = MBI.db.profile.general.layout.textSize * mult
	local r, g, b = GetColor(MBI.db.profile.threshold[igtype].color)

	-- background
	if MBI.db.profile.ignite.layout.background.enabled then
		local rb, gb, bb = r * MBI.db.profile.ignite.layout.background.darkness, g * MBI.db.profile.ignite.layout.background.darkness, b * MBI.db.profile.ignite.layout.background.darkness
		MBI.F.main:SetBackdropColor(rb, gb, bb)
	end
	-- border
	if MBI.db.profile.ignite.layout.border.enabled then
		MBI.F.main:SetBackdropBorderColor(r, g, b)
	end
	-- dmg
	MBI.F.timer.txt:SetFont(MBI.font, txtSize)
	MBI.F.timer.txt:SetTextColor(r, g, b)

	-- frame size
	if MBI.db.profile.ignite.power.enabled then
		MBI.F.timer:SetWidth(MBI.F.timer.w * mult)
	end
	MBI.F.main:SetWidth(MBI.F.main.w * mult)
	MBI.F.main:SetHeight(MBI.F.main.h * mult)

	-- ignite timer ticks
	if MBI.db.profile.ignite.timer.tick then
		for i = 0, 2 do
			MBI.F.tick[i]:SetWidth(MBI.F.tick.size * mult)
			MBI.F.tick[i]:SetHeight(MBI.F.tick.size * mult)
			MBI.F.tick[i]:ClearAllPoints()
			if not MBI.db.profile.ignite.timer.reverse then
				MBI.F.tick[i]:SetPoint("BOTTOMLEFT", MBI.F.timer, "BOTTOMLEFT", i / 3 * MBI.F.timer.w * mult, 0)
			else
				MBI.F.tick[i]:SetPoint("BOTTOMRIGHT", MBI.F.timer, "BOTTOMRIGHT", -i / 3 * MBI.F.timer.w * mult, 0)
			end
			MBI.F.tick[i]:SetBackdropColor(r, g, b, MBI.db.profile.ignite.timer.alpha)
		end
	end
end

-- ignite timer bar update
function MBI:UpdateIgniteTimerDisplay(igtype, igremain)
	local r, g, b
	if igremain > MBI.db.profile.ignite.timer.redZone then
		r, g, b = GetColor(MBI.db.profile.threshold[igtype].color)
	else
		r, g, b = GetColor(MBI.db.profile.ignite.timer.redZoneColor)
	end
	MBI.F.timer:SetStatusBarColor(r, g, b, MBI.db.profile.ignite.timer.alpha)
end

-- ignite power bar update
function MBI:UpdateIgnitePowerDisplay(value)
	local r, g, b = 0, 0, 0
	local c1, c2 = 0, 0

	if value > MBI.db.profile.ignite.power.middle then
		c1 = 1 - (value - MBI.db.profile.ignite.power.middle) / (1 - MBI.db.profile.ignite.power.middle)
		c2 = 1
	else
		c1 = 1
		c2 = 1 + (value - MBI.db.profile.ignite.power.middle) / MBI.db.profile.ignite.power.middle
	end
	if MBI.db.profile.ignite.power.gradient == "rg" then
		r, g = c1, c2
	else
		b, g = c1, c2
	end
	
	MBI.F.power:SetStatusBarColor(r, g, b, MBI.db.profile.ignite.power.alpha)
end



--##############################################################################################################################################
-- core functions-------------------------------------------------------------------------------------------------------------------------------

-- combat log
function MBI:CombatLog(event, ...)
	local _, eventType, _, sourceGUID, _, _, _, destGUID, _, _, _, spellID, _, _, amount = ...
	
	if sourceGUID == UnitGUID("player") then
		-- ignite damages
		if string.find(eventType, "_DAMAGE") and spellID == MBI.spellID.ignite then
			local igDmg = amount

			-- target specific ignite damages
			if destGUID == UnitGUID("target") then
				-- synchro of smooth dmg display with ignite tick
				if MBI.db.profile.ignite.smoothDmgReport and not MBI.db.profile.ignite.useCombatLog then
					MBI.smoothDmg = MBI.db.profile.ignite.smoothDmgValue
				end
	
				-- ig dmg and power bar value determined by combat event
				if MBI.db.profile.ignite.useCombatLog then
					MBI.igDmg = igDmg
					if MBI.db.profile.ignite.power.enabled then
						MBI.F.power.endValue = igDmg / (MBI.power.average + MBI.power.sigma * 3)
					end	
				end
			end
			
			-- stat module
			if igDmg ~= 0 then
				if igDmg > MBI.db.profile.stats.info.highest then
					MBI.db.profile.stats.info.highest = igDmg
				elseif igDmg < MBI.db.profile.stats.info.lowest then
					MBI.db.profile.stats.info.lowest = igDmg
				end
				if igDmg > MBI.db.profile.threshold.value then
					MBI.db.profile.stats.info.supOptT = MBI.db.profile.stats.info.supOptT + 1
				elseif igDmg > MBI.db.profile.threshold.value * MBI.db.profile.threshold.interm.perct then
					MBI.db.profile.stats.info.supIntermT = MBI.db.profile.stats.info.supIntermT + 1
				end
				
				MBI.db.profile.stats.info.dmg = MBI.db.profile.stats.info.dmg + igDmg
				MBI.db.profile.stats.info.count = MBI.db.profile.stats.info.count + 1
				MBI.db.profile.stats.info.thresholdCount = MBI.db.profile.stats.info.thresholdCount + 1
				
				MBI.db.profile.stats.info.average = MBI.db.profile.stats.info.dmg / MBI.db.profile.stats.info.count
				
				if MBI.db.profile.stats.info.count > MBI.db.profile.general.resetStats * 10^6 then
					wipe(MBI.db.profile.stats.info)
					MBI:InitStatModule()
				end
			end

			-- ignite powerbar stats
			if MBI.db.profile.ignite.power.enabled then
				if igDmg ~= 0 then
					tinsert(MBI.db.profile.stats.power, igDmg)
					if #MBI.db.profile.stats.power > MBI.db.profile.ignite.power.tableSize then
						tremove(MBI.db.profile.stats.power, 1)
					end
					MBI:CalculatePower()
				end
			end
		end
	end	
end

-- power bar numbers update
function MBI:CalculatePower()
	local average = 0
	local sigma = 0
	for i = 1, #MBI.db.profile.stats.power do
		average = average + MBI.db.profile.stats.power[i]
	end
	average = average / #MBI.db.profile.stats.power
	for i = 1, #MBI.db.profile.stats.power do
		sigma = sigma + (MBI.db.profile.stats.power[i] - average)^2
	end
	sigma = math.sqrt(sigma / #MBI.db.profile.stats.power)
	
	MBI.power.average = average
	MBI.power.sigma = sigma
end

-- main engine : aura tracking / damage fetching / timers / ...
function MBI:MainEngine(elapsed)
	self.TimeSinceLastUpdate = (self.TimeSinceLastUpdate or 0) + elapsed
	if self.TimeSinceLastUpdate > MBI.REFRESH then
		local time = GetTime()
		local igType = "low"
		local isDotAbsent = false
		local isTarget = UnitGUID("target")
		local igRemain, igDmg = 0, 0

		-- scanning of target for ignite and pyro
		for i = 1, #MBI.dotToTrack do
			local _, _, _, _, _, duration, expTime, _, _, _, _, _, _, val1 = UnitDebuff("target", MBI.dotToTrack[i], nil, "PLAYER")

			-- determine time remaining specifically for a given dot tracked
			if expTime ~= nil then
				MBI.remainTime[i] = expTime - time
				if MBI.remainTime[i] <= 0 then
					MBI.remainTime[i] = 0
				end
			else
				MBI.remainTime[i] = 0
			end

			-- checks if time > refresh for each DoT + handling of redZone and icon
			if MBI.remainTime[i] < MBI.REFRESH then
				isDotAbsent = true
			end

			-- ignite dmg display coupled with on update
			if MBI.dotToTrack[i] == MBI.dotName.ignite then
				igRemain = MBI.remainTime[i]
				MBI.smoothDmg = MBI.smoothDmg + MBI.REFRESH

				-- frame fading depending ignite remaining time
				if MBI.remainTime[i] < MBI.REFRESH and MBI.db.profile.general.autohide.noIgnite then
					MBI.F.main:Hide()
				else
					MBI.F.main:Show()
				end

				-- ignite damage
				if MBI.remainTime[i] < MBI.REFRESH or not MBI.inCombat or not val1 then
					igDmg = 0
					igRemain = 0
					MBI.igDmg = 0
					MBI.smoothDmg = 0
				elseif MBI.db.profile.ignite.useCombatLog then
					igDmg = MBI.igDmg
				elseif MBI.db.profile.ignite.smoothDmgReport then
					if MBI.smoothDmg >= MBI.db.profile.ignite.smoothDmgValue then
						MBI.igDmg = val1
						MBI.smoothDmg = 0
					end
					igDmg = MBI.igDmg
				else
					igDmg = val1
				end
			end
		end

		-- ignite dmg report
		if MBI.db.profile.ignite.dmgAsDps then
			MBI.F.timer.txt:SetText(FormatText(Round(igDmg / 2, 0)))
		else
			MBI.F.timer.txt:SetText(FormatText(igDmg))
		end

		-- power bar
		if MBI.db.profile.ignite.power.enabled then
			MBI.F.power.endValue = igDmg / (MBI.power.average + MBI.power.sigma * 3)
			if MBI.F.power.endValue > 1 then
				MBI.F.power.endValue = 1
			end
		end
		
		-- get combustion CD remaining
		MBI:CombuCheck()

		-- updates frame display in function of DoTs / ignite dmg
		if not MBI.combuCD and not isDotAbsent then
			if igDmg >= MBI.db.profile.threshold.value then
				igType = "high"
			elseif igDmg >= MBI.db.profile.threshold.value * MBI.db.profile.threshold.interm.perct and MBI.db.profile.threshold.interm.enabled then
				igType = "mid"
			end
		end
		MBI:UpdateIgniteFrame(igType)
		
		-- timer bar
		if MBI.db.profile.ignite.timer.enabled then
			if igRemain > MBI.REFRESH then
				MBI.F.timer.value = igRemain
				MBI.F.timer:SetValue(igRemain)
				MBI:UpdateIgniteTimerDisplay(igType, igRemain)
			else
				MBI.F.timer.value = 0
				MBI.F.timer:SetValue(0)
				MBI:UpdateIgniteTimerDisplay("low", 0)
			end
		end
		
		-- frame hiding if display w/o ignite is enabled but player has no target
		if not isTarget and MBI.db.profile.general.autohide.noCombat then
			MBI.F.main:Hide()
		end
		-- frame hiding if combu in CD and frame hiding enabled
		if MBI.db.profile.general.autohide.combuCD and MBI.combuCD and MBI.combuCDTime > MBI.db.profile.general.autohide.combuCDTime then
			MBI.F.main:Hide()
		end
		
		-- stops the on update code after 5 sec out of combat / no combustion in CD
		if not MBI.inCombat then
			if MBI.db.profile.general.autohide.noCombat and not MBI.cfMode then
				MBI.F.main:Hide()
			else
				MBI.F.main:Show()
			end
			MBI:UpdateIgniteFrame(igType)
			if not MBI.combuCD then
				MBI.time_before_stop_onupdate = MBI.time_before_stop_onupdate - MBI.REFRESH
				if MBI.time_before_stop_onupdate < 0 then
					MBI.F.combat:Hide()
					MBI:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
					MBI.time_before_stop_onupdate = 10
					MBI.smoothDmg = 0
				end
			end
		end

		self.TimeSinceLastUpdate = 0
	end
end



--##############################################################################################################################################
-- helper functions ----------------------------------------------------------------------------------------------------------------------------

-- UI scale change detection
function MBI:ScaleChange()
	local UIScale = UIParent:GetScale()
	MBI.db.profile.general.anchor.coord.xcd = MBI.F.anchor:GetLeft() * UIScale
	MBI.db.profile.general.anchor.coord.ycd = MBI.F.anchor:GetBottom() * UIScale
	if MBI.configLoaded then
		wipe(MBI.options.args)
		MBI:LoadFireOptions()
		MBI:ChangeConfigDisplay("fireSpec")
	end
end

-- combustion CD check function
function MBI:CombuCheck()
	local startCD, durationCD = GetSpellCooldown(MBI.spellID.combustion)
	local time = GetTime()
	if time > startCD + durationCD then
		MBI.combuCD = false
		MBI.combuCDTime = 0
	else
		MBI.combuCD = true
		MBI.combuCDTime = startCD + durationCD - time
		MBI.F.combat:Show()
	end
end

-- combat check function
function MBI:CombatCheck(event)
	if event == "PLAYER_REGEN_DISABLED" then
		MBI.F.combat:Show()
		MBI:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLog")
		if MBI.F.info and MBI.F.info:IsShown() then
			MBI.F.info:Hide()
		end
		MBI.inCombat = true
		ACR:NotifyChange("myBigIgnite")
	else
		MBI.inCombat = false
		ACR:NotifyChange("myBigIgnite")
		if MBI.db.profile.threshold.value == 0 then
			print(MBI.ORANGE .. L["myBigIgnite: "] .. MBI.YELLOW_LIGHT .. L["notyetthreshold"])
			UIErrorsFrame:AddMessage(MBI.ORANGE .. L["notyetthreshold"])
		end
	end
end

-- command parser
function MBI:CmdParser(cmd)
	if cmd == "config" then
		MBI:OpenConfig()
	elseif string.find(cmd, "threshold") then
		local threshold = string.match(cmd, " (%d+)")
		if threshold then
			MBI.db.profile.threshold.value = tonumber(string.match(cmd, " (%d+)"))
			MBI.db.profile.stats.info.thresholdCount = 0
			MBI.db.profile.stats.info.supOptT = 0
			MBI.db.profile.stats.info.supIntermT = 0
			if MBI.F.info and MBI.F.info:IsShown() and MBI.infoTable.type == "stats" then
				MBI:PopulateInfo("stats")
				MBI:InfoScrollUpdate()
			end
			ACR:NotifyChange("myBigIgnite")
		else
			print(format(L["%senter a positive number"], HlString(L["myBigIgnite: "])))
		end
	elseif cmd == "help" or cmd == "stats" then
		MBI:OpenInfo(cmd, true)
	else
		print(MBI.ORANGE .. "myBigIgnite v" .. MBI.VERSION .. " | Istaran - Medivh EU.")
		print(MBI.YELLOW_LIGHT .. L["/mbi help to obtain infos."])
	end
end



------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------

------------------OLD STUFF

------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------




	--if not MBI.addonLoadded then
		--_, _, MBI.combuIcon = GetSpellInfo(MBI.spellID.combustion)
		--_, _, MBI.fbIcon = GetSpellInfo(MBI.spellID.fireball)
		--_, _, MBI.igniteIcon = GetSpellInfo(MBI.spellID.ignite)
		--_, _, MBI.pyroIcon = GetSpellInfo(MBI.spellID.pyroblast)
		--MBI.infernoName, _, MBI.infernoIcon = GetSpellInfo(MBI.spellID.inferno)
		-- _, _, MBI.lbIcon = GetSpellInfo(MBI.spellID.livingBomb)
	--end




		--elseif MBI.combuCD and MBI.db.profile.ignite.impactMode and MBI.impactKnown and isImpact and not isDotAbsent then
		--	if igDmg >= MBI.db.profile.threshold.value * MBI.db.profile.values.impactPerct and MBI.db.profile.values.impactPerctEnabled then
		--		igType = "inferno"
		--	end
		--	if MBI.db.profile.icons.impactIcon then
		--		spellIconPath = MBI.fireblastIcon
		--	end


			--elseif redZone then
			--	igType = "rz"
			--	if MBI.db.profile.ignite.impactMode and MBI.impactKnown and isImpact then
			--		if igDmg >= MBI.db.profile.threshold.value * MBI.db.profile.values.impactPerct and MBI.db.profile.values.impactPerctEnabled then
			--			igType = "inferno"
			--		end
					--if MBI.db.profile.icons.impactIcon then
						--spellIconPath = MBI.fireblastIcon
					--end
				--end




		-- scaning of player for heating up and pyro!
		-- if MBI.db.profile.ignite.impactMode and MBI.impactKnown then									-- Impact tracking
			-- local _, _, _, _, _, _, expTimeImpact = UnitAura("player", MBI.impactName, nil, "PLAYER|HELPFULL")
			-- if expTimeImpact ~= nil then
				-- remainTimeImpact = expTimeImpact - time
				-- if remainTimeImpact <= 0 then
					-- remainTimeImpact = 0
				-- end
			-- end

			-- if remainTimeImpact > MBI.REFRESH then
				-- isImpact = true
			-- end
		-- end
		-------------





-- normal or zoomed icon
--function MBI:SetIconType(frame, iconType)
--	MBI.F[frame].tt:ClearAllPoints()
--	if string.find(iconType, "zoom") then
--		MBI.F[frame].tt:SetTexCoord(0.08, 0.92, 0.08, 0.92)
--	else
--		MBI.F[frame].tt:SetTexCoord(0, 1, 0, 1)
--	end
--	MBI.F[frame].tt:SetAllPoints(MBI.F.icon)
--end

-- icon border color
--function MBI:SetIconBorderColor(t)
--	MBI.F.icon:SetBackdropBorderColor(GetColor(t))
--end

-- accessory creation of icon frame / allow on demand creation
--function MBI:CreateIconFrame()
--	MBI.F.icon = CreateFrame("Frame", "MBI.F.icon", MBI.F.main)
--	MBI:SetFrameSize("icon")
--	MBI:SetFrameInteractive("icon", false, false)
--	MBI:SetFramePoint("icon", MBI.db.profile.icons.side, MBI.F.main, MBI.db.profile.icons.side, MBI.F.icon.off, 0)
--	MBI.F.icon.tt = MBI.F.icon:CreateTexture("MBI.F.icon.tt", "BACKGRound")
--	MBI:SetIconType("icon", MBI.db.profile.icons.type)
--	MBI.F.icon.tt:SetTexture(MBI.igniteIcon)
--	MBI:SetFrameBackdrop("icon")
--	MBI:SetIconBorderColor(MBI.db.profile.threshold.low.bd)
--end





	--local isVisualRZ = false
	--local iconPath = iconpath or MBI.lbIcon

	-- red zone handling, color of each type of display is of the red zone if visual red zone is enabled. color of calor tables is changed here
	--if redzone and igtype ~= "low" then
	--	if MBI.db.profile.ignite.lagVisual then
	--		for i = 1, #MBI.elem do
	--			if MBI.db.profile.ignite.lagVisualType[MBI.elem[i]] then
	--				for k, v in pairs(MBI[igtype][MBI.elem[i]]) do
	--					if MBI.db.profile.ignite.lagColor[k] ~= nil then
	--						MBI[igtype][MBI.elem[i]][k] = MBI.db.profile.ignite.lagColor[k]
	--					end
	--				end
	--			end
	--		end
	--		isVisualRZ = true
	--	else
	--		-- no visual red zone
	--		MBI:UpdateIgniteFrame("low", iconPath)
	--		return
	--	end
	--end






	 --back to default state when red zone is over and visual red zone is enabled
	--if MBI.wasVisualRZ and not isVisualRZ then
	--	for i = 1, #MBI.elem do
	--		if MBI.db.profile.ignite.lagVisualType[MBI.elem[i]] then
	--			for k, v in pairs(MBI[igtype][MBI.elem[i]]) do
	--				MBI[igtype][MBI.elem[i]][k] = MBI.db.profile.threshold[igtype][MBI.elem[i]][k]
	--			end
	--		end
	--	end
	--end

	-- icon frame
	--if MBI.db.profile.icons.enabled then
	--	MBI.F.icon:SetWidth(MBI.F.icon.w * mult)
	--	MBI.F.icon:SetHeight(MBI.F.icon.w * mult)
	--	MBI:SetFramePoint("icon", MBI.db.profile.icons.side, MBI.F.main, MBI.db.profile.icons.side, MBI.F.icon.off * mult, 0)
	--	if string.find(MBI.db.profile.icons.type, "bord") then
	--		MBI:SetIconBorderColor(r, g, b)
	--	end
	--	MBI.F.icon.tt:SetTexture(iconPath)
	--end

	-- ignite timer
	--if MBI.db.profile.ignite.timer.enabled then
	--	MBI.F.timer:SetHeight(MBI.F.timer.h * mult)
	--	MBI.F.timer:SetWidth(MBI.F.timer.w * mult)
	--end







--
--
--			--icons = {
			--	enabled = false,
			--	side = "LEFT",
			--	sizeFactor = 1.35,
			--	type = "zoombord",
			--	bdSize = 0.75,
			--	combuCD = {
			--		enabled = true,
			--		side = "LEFT",
			--	},
			--	heatingup = {
			--		enabled = true,
			--		side = "LEFT",
			--	},
			--	bomb = {
			--		enabled = true,
			--		side = "LEFT",
			--	},
			--},

--
--
--
--
--
-- color tables for each type of display
--function MBI:CreateColorTables()
--	MBI.igType = {
--		[1] = "high",
--		[2] = "mid",
--		[3] = "low",
--	}
--	MBI.elem = {
--		[1] = "text",
--		[2] = "bg",
--		[3] = "bd",
--	}
--	for i = 1, #MBI.igType do
--		MBI[MBI.igType[i]] = {}
--		for j = 1, #MBI.elem do
--			MBI[MBI.igType[i]][MBI.elem[j]] = {}
--			for k, v in pairs(MBI.db.profile.emphasis[MBI.igType[i]][MBI.elem[j]]) do
--				MBI[MBI.igType[i]][MBI.elem[j]][k] = MBI.db.profile.emphasis[MBI.igType[i]][MBI.elem[j]][k]
--			end
--		end
--	end
--end
--
--
--
--
--
--
--
--
--
--
--
--
--
--
-- sets the size of each frame (called using frame name as frame)
--function MBI:SetFrameSize(frame)
--	local width, height = 1, 1
--	if frame == "dmg" then
--		MBI.F[frame].txt:SetFont(MBI.font, MBI.db.profile.general.layout.textSize)
--		MBI.F[frame].txt:SetShadowColor(0, 0, 0, 1)
--		MBI.F[frame].txt:SetShadowOffset(1, -1)
--		MBI:AlignText("dmg", MBI.db.profile.ignite.layout.text.align, MBI.db.profile.ignite.layout.text.align, MBI.db.profile.ignite.layout.text.ofx, MBI.db.profile.ignite.layout.textOffsetY)
--		MBI.F[frame].txt:SetText(FormatText(999999))
--		width = MBI.F[frame].txt:GetStringWidth() * 1.2
--		height = MBI.F[frame].txt:GetStringHeight() * 1.4
--		MBI.F.timer.w = width
--		MBI.F.timer.h = height
--		MBI.F.timer.txt:SetText(FormatText(0))
--	elseif frame == "icon" then
--		height = MBI.F.timer.h
--		width = height * MBI.db.profile.icons.sizeFactor
--		MBI.F.icon.off = (height - width) / 2
--		MBI.F.icon.w = width
--		height = width
--	elseif frame == "main" then
--		width = MBI.F.timer.w * 1.25
--		height = MBI.F.timer.h
--		MBI.F.main.w = width
--		MBI.F.main.h = height
--	end
--	MBI.F[frame]:SetWidth(width)
--	MBI.F[frame]:SetHeight(height)
--end
--
-- sets frame backdrops
--function MBI:SetFrameBackdrop(frame)
--	if MBI.F[frame] then
--		local backdrop = {
--			bgFile = LSM:Fetch("background", MBI.db.profile[frame].bgTexture) or "",
--			edgeFile = LSM:Fetch("border", MBI.db.profile[frame].bdTexture) or "",
--			edgeSize = MBI.db.profile[frame].bdSize,
--			insets = {
--				left = MBI.db.profile[frame].inset,
--				right = MBI.db.profile[frame].inset,
--				top = MBI.db.profile[frame].inset,
--				bottom = MBI.db.profile[frame].inset,
--			} or "",
--		}
--
--		if frame == "main" and not MBI.db.profile[frame].bgEnabled then
--			backdrop.bgFile = ""
--		end
--		if frame == "main" and not MBI.db.profile[frame].bdEnabled then
--			backdrop.edgeFile = ""
--		end
--		if frame == "icon" then
--			if string.find(MBI.db.profile[frame].type, "bord") then
--			backdrop.edgeFile = LSM:Fetch("border", "Solid")
--			else
--				backdrop.edgeFile = ""
--			end
--		end
--		MBI.F[frame]:SetBackdrop(backdrop)
--	end
--end
--
-- anchor frame strata
--function MBI:SetAnchorFrameStrata()
--	if MBI.db.profile.general.anchor.strata == 1 then
--		MBI.F.anchor:SetFrameStrata("LOW")
--	elseif MBI.db.profile.general.anchor.strata == 2 then
--		MBI.F.anchor:SetFrameStrata("MEDIUM")
--	else
--		MBI.F.anchor:SetFrameStrata("HIGH")
--	end
--end
--
-- anchor frame position
--function MBI:SetAnchorFramePosition()
--	local UIScale = UIParent:GetScale()
--	local UIWidth = UIParent:GetWidth()
--	local UIHeight = UIParent:GetHeight()
--	MBI.F.anchor:ClearAllPoints()
--	MBI.F.anchor:SetPoint("CENTER", UIParent, "CENTER", MBI.db.profile.general.anchor.coord.xcd / UIScale - UIWidth / 2 , MBI.db.profile.general.anchor.coord.ycd / UIScale - UIHeight / 2)
--end
--
-- sets frame anchors
--function MBI:SetFramePoint(frame, ...)
--	local point, parent, relpoint, xcd, ycd = ...
--	MBI.F[frame]:ClearAllPoints()
--
--	MBI.F[frame]:SetPoint(point, parent, relpoint, xcd, ycd)
--
--	if frame == "dmg" then
--		if point == "TOPLEFT" then
--			point = "BOTTOMLEFT"
--			relpoint = "BOTTOMLEFT"
--			ycd = -ycd
--		else
--			point = "BOTTOMRIGHT"
--			relpoint = "BOTTOMRIGHT"
--			ycd = -ycd
--		end
--		MBI.F[frame]:SetPoint(point, parent, relpoint, xcd, ycd)
--	end
--end
--
-- text anchoring inside dmg frame
--function MBI:AlignText(frame, ...)
--	MBI.F[frame].txt:ClearAllPoints()
--	local point, relpoint, ofx, ofy = ...
--	if MBI.db.profile.ignite.layout.text.align == "CENTER" then
--		ofx = 0
--		ofy = 0
--	end
--	if point == "RIGHT" then
--		ofx = -ofx
--	end
--	MBI.F[frame].txt:SetPoint(point, MBI.F[frame], relpoint, ofx, ofy)
--end
--
--
--
--
-- ignite timer bar
--function MBI:SetIgniteTimer()
--	local frame = MBI.F.timer
--
--	frame:ClearAllPoints()
--	frame:SetStatusBarTexture(LSM:Fetch("statusbar", MBI.db.profile.general.layout.bar))
--	frame:SetPoint("TOPLEFT", MBI.F.timer, "TOPLEFT")
--	frame:SetPoint("BOTTOMRIGHT", MBI.F.timer, "BOTTOMRIGHT")
--	frame:SetReverseFill(MBI.db.profile.ignite.timer.reverse)
--	frame:SetWidth(1)
--	frame:SetHeight(1)
--	frame:SetMinMaxValues(0, 6)
--	frame:SetValue(0)
--
--	 on update code for the bar
--	frame:SetScript("OnUpdate", function(_, elapsed)
--		if frame.value and frame.value > 0 then
--			frame.value = frame.value - elapsed
--			frame:SetValue(frame.value)
--		end
--	end)
--
--	MBI:SetTickMarks()
--
--	frame:Hide()
--	if MBI.cfMode then
--		frame:SetValue(6)
--	end
--end
--
--
--function MBI:SetTickMarks()
--
--	MBI.F.tick1:ClearAllPoints()
--	local height = 0.5 * (MBI.F.main.h - 2 * MBI.db.profile.ignite.ignite.layout.adjust)
--	local width = height * 0.4
--	MBI.F.tick1:SetHeight(height)
--	MBI.F.tick1:SetWidth(width)
--	MBI.F.tick1:SetPoint("BOTTOM", MBI.F.tick, "BOTTOM")
--	MBI.F.tick1:SetBackdrop({bgFile = LSM:Fetch("border", "Blizzard Tooltip")})
--	MBI.F.tick1:SetBackdropColor(GetColor(MBI.db.profile.ignite.timer.tickColor))
--
--	MBI.F.tick2:ClearAllPoints()
--	local height = 0.5 * (MBI.F.main.h - 2 * MBI.db.profile.ignite.ignite.layout.adjust)
--	local width = height * 0.4
--	MBI.F.tick2:SetHeight(height)
--	MBI.F.tick2:SetWidth(width)
--	MBI.F.tick2:SetPoint("BOTTOM", MBI.F.tick, "BOTTOM", 5, 0)
--	MBI.F.tick2:SetBackdrop({bgFile = LSM:Fetch("border", "Blizzard Tooltip")})
--	MBI.F.tick2:SetBackdropColor(GetColor(MBI.db.profile.ignite.timer.tickColor))
--
--
--
--end
--
-- mouse interaction
--function MBI:SetFrameInteractive(frame, movable, mouse)
--	MBI.F[frame]:SetMovable(movable)
--	MBI.F[frame]:EnableMouse(mouse)
--end
